/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)cdr_typeObject.h    generated by: makeheader    Sat Mar  9 09:37:57 2024
 *
 *		built from:	typeObject.ifc
 */

#ifndef cdr_typeObject_h
#define cdr_typeObject_h


 #ifndef osapi_type_h
    #include "osapi/osapi_type.h"
 #endif
 #ifndef cdr_dll_h
    #include "cdr/cdr_dll.h"
 #endif
 #ifndef cdr_type_h
    #include "cdr/cdr_type.h"
 #endif
#ifndef cdr_stream_h
    #include "cdr/cdr_stream.h"
#endif
#ifndef reda_fastBuffer_h
    #include "reda/reda_fastBuffer.h"
#endif
#ifndef cdr_typecode_h
    #include "cdr/cdr_typeCode.h"
#endif


#ifdef __cplusplus
    extern "C" {
#endif


struct RTICdrTypeObject;

typedef struct RTICdrTypeObjectFactory {

    /*i @brief (private) Fast buffer pool to allocate TypeObjects in buffer
       form when size is below deserializedAllocThreshold
      */
    struct REDAFastBufferPool * _deserializePool;
    
    /*i @brief (private) Fast buffer pool to allocate serialize buffers when.
     * size is below serializedAllocThreshold.
     *
     * Used to generate the buffers returned by
     * RTICdrTypeObjectFactory_getSerializationBuffer.
     */
    struct REDAFastBufferPool * _serializePool;

    /*i @brief (read-only) */
    int maxSerializedSize;
    /*i @brief (read-only) */
    int maxDeserializedSize;
    /*i @brief (read-only) */
    int serializedAllocThreshold;
    /*i @brief (read-only) */
    int deserializedAllocThreshold;

} RTICdrTypeObjectFactory;

extern RTICdrDllExport 
RTICdrTypeObjectFactory * RTICdrTypeObjectFactory_new(
    int maxSizeSerialized,
    int maxSizeDeserialized,
    int serializedDynamicAllocationThreshold,
    int deserializedDynamicAllocationThreshold,
    struct REDAFastBufferPoolProperty * poolProperty);

extern RTICdrDllExport
void *RTICdrTypeObjectFactory_getDeserializationBuffer(
    struct RTICdrTypeObjectFactory *self,
    int deserializedSize);

extern RTICdrDllExport
void RTICdrTypeObjectFactory_returnDeserializationBuffer(
    struct RTICdrTypeObjectFactory *self,
    void *buffer,
    int bufferSize);

extern RTICdrDllExport
void *RTICdrTypeObjectFactory_getSerializationBuffer(
        struct RTICdrTypeObjectFactory *self,
        int serializedSize);

extern RTICdrDllExport
void RTICdrTypeObjectFactory_returnSerializationBuffer(
    struct RTICdrTypeObjectFactory *self,
    void *buffer,
    int bufferSize);

extern RTICdrDllExport 
void RTICdrTypeObjectFactory_delete(
    RTICdrTypeObjectFactory * self);

extern RTICdrDllExport 
struct RTICdrTypeObject * RTICdrTypeObjectFactory_createTypeObjectFromTypeCode(
    RTICdrTypeObjectFactory * self,
    const RTICdrTypeCode * typeCode);

extern RTICdrDllExport 
struct RTICdrTypeObject * RTICdrTypeObjectFactory_createTypeObjectBufferFromTypeCode(
    RTICdrTypeObjectFactory * self,
    RTICdrOctet * typeObjectMd5, /* out */
    const RTICdrTypeCode * typeCode);

extern RTICdrDllExport 
struct RTICdrTypeObject * RTICdrTypeObjectFactory_createTypeObjectBuffer(
    RTICdrTypeObjectFactory * self,
    int deserializedSize,
    RTIBool forceDynamicAllocation);

extern RTICdrDllExport
struct RTICdrTypeObject * RTICdrTypeObjectFactory_deserializeTypeObject(
    RTICdrTypeObjectFactory * self,
    RTICdrOctet * typeObjectMd5,
    struct RTICdrStream * stream,
    void * endpointData,
    void * endpointPluginQos);

extern RTICdrDllExport  
struct RTICdrTypeObject * RTICdrTypeObjectFactory_copyTypeObject(
    struct RTICdrTypeObjectFactory * self,
    const struct RTICdrTypeObject * source,
    RTIBool forceDynamicAllocation);

extern RTICdrDllExport 
RTIBool RTICdrTypeObjectFactory_deleteTypeObject(
    struct RTICdrTypeObjectFactory * self,
    const struct RTICdrTypeObject * typeObject);

extern RTICdrDllExport 
int RTICdrTypeObjectFactory_getTypeObjectDeserializedSize(
    struct RTICdrTypeObjectFactory * self,
    const struct RTICdrTypeObject * typeObject);


struct RTICdrTypeObjectAssignabilityProperty {
    /*e
     * When set to true, the assignability algorithm won't enforce the rule
     * that says that members of aggregation types (struct, union) with
     * the same ID must have the same name.
     *
     * For example, A and B would be assignable only if ignoreMemberNames is set
     * to true:
     *
     * struct A {
     *    long a;
     * };
     *
     * struct B {
     *    long b;
     * };
     */
    RTIBool ignoreMemberNames;

    /*e
     * When set to true, the assignability algorithm won't enforce the rule
     * that says that any enum literals that have the same value must also have
     * the same name.
     *
     * For example, A and B would be assignable only if ignoreEnumLiteralNames
     * is set to true:
     *
     * enum Color {
     *    ROJO = 1
     * };
     *
     * enum Color {
     *    RED = 1
     * };
     */
    RTIBool ignoreEnumLiteralNames;

    /*e
     * When set to true, the assignability algorithm won't enforce the rule
     * that disallows assigning a sequence to another sequence with a smaller
     * bound.
     *
     * For example, sequence<long, 100> is assignable to sequence<long, 200> but
     * sequence<long, 200> is only assignable to sequence<long, 100> if
     * ignoreSequenceBounds is true.
     *
     * This property applies to sequence, string and map types. It does not
     * affect arrays, whose bounds must be equal in all cases.
     *
     * Note: the deserialization algorithm takes into can handle on a per-sample
     * basis objects that are not assignable, depending on the actual sequence
     * length in that sample.
     */
    RTIBool ignoreSequenceBounds;

    /*e
     * When set to true, the assignability algorithm won't enforce the rule
     * that disallows assigning a string to another string with a smaller
     * bound.
     *
     * For example, string<100> is assignable to string<200> but
     * string<200> is only assignable to string<100> if
     * ignoreStringBounds is true.
     */
    RTIBool ignoreStringBounds;

    /*e
     * When set to true, the assignability algorithm will enforce the rule
     * that disallows assigning a wider type to a narrower type. A type T2 is
     * said to widen type T1 when type T2 contains non-optional fields that are
     * not present in T1.
     */
    RTIBool preventTypeWidening;

    /*e
     * When set to true, the assignability algorithm will consider the type to
     * be serialized using XCDR encoding version 2.
     */
    RTIBool useXcdr2;

    /*
     * Indicates whether or not a bound of zero should be treated as unbounded
     *
     * This boolean will be set to true when the TypeObject is received from
     * a different vendor because according to the XTypes specification,
     * the bounds for unbounded sequences and strings should be represented
     * as 0.
     *
     * Our own implementation does not use 0 to represent unbounded on the
     * TypeObject but other vendors do.
     */
    RTIBool zeroBoundAsUnbounded;
};


#define RTI_CDR_TYPE_OBJECT_ASSIGNABILITY_PROPERTY_DEFAULT {                  \
    RTI_FALSE /* ignoreMemberNames */,                                        \
    RTI_FALSE /* ignoreEnumLiteralNames */,                                   \
    RTI_FALSE /* ignoreSequenceBounds */,                                     \
    RTI_FALSE /* ignoreStringBounds */,                                       \
    RTI_FALSE /* preventTypeWidening */,                                      \
    RTI_FALSE /* useXcdr2 */,                                                 \
    RTI_FALSE /* zeroBoundAsUnbounded */                                      \
}

extern RTICdrDllExport RTIBool RTICdrTypeObject_serialize(
    void *endpointData,
    const struct RTICdrTypeObject *typeObject,
    struct RTICdrStream *stream,
    RTIBool serializeEncapsulation, 
    RTIEncapsulationId encapsulationId,
    RTIBool serializeSample,
    void *endpointPluginQos);

extern RTICdrDllExport
RTIBool RTICdrTypeObject_equals(const struct RTICdrTypeObject * self,
                                const struct RTICdrTypeObject * other);

extern RTICdrDllExport
RTIBool RTICdrTypeObject_equals_w_params(
    const struct RTICdrTypeObject * self,
    const struct RTICdrTypeObject * other,
    RTIBool printCause);

extern RTICdrDllExport RTIBool
RTICdrTypeObject_is_assignable(const struct RTICdrTypeObject * self,
                               const struct RTICdrTypeObject * other);

extern RTICdrDllExport RTIBool
RTICdrTypeObject_is_assignable_w_property(
    const struct RTICdrTypeObject * self,
    const struct RTICdrTypeObject * other,
    const struct RTICdrTypeObjectAssignabilityProperty * property);

extern RTICdrDllExport struct RTICdrTypeObject *
RTICdrTypeObject_createFromTypeCode(struct RTICdrTypeObject *self,
                                    RTICdrTypeCode *typeCode);


struct RTICdrTypeObjectToTypeCodeProperty {
    /**
     * @brief When set to true, a bound 0 for sequences and strings will be
     * translated to unbounded when converting from TypeObjects to TypeCodes.
     *
     * This boolean will be set to true when the TypeObject is received from 
     * a different vendor because according to the XTypes specification,
     * the bounds for unbounded sequences and strings should be represented
     * as 0.
     *
     * Our own implementation does not use 0 to represent unbounded on the 
     * TypeObject but other vendors do.
     *
     * See CORE-14185.
     */
    RTIBool zeroBoundAsUnbounded;
};


#define RTICdrTypeObjectToTypeCodeProperty_INITIALIZER { \
    RTI_FALSE /* zeroBoundAsUnbounded */ \
}

extern RTICdrDllExport RTIBool RTICdrTypeObject_convertToTypeCode(
        struct RTICdrTypeObject *self,
        RTICdrTypeCode **typeCode,
        const struct RTICdrTypeObjectToTypeCodeProperty *property);

extern RTICdrDllExport void
RTICdrTypeObject_destroyTypeObject(struct RTICdrTypeObject *self);

extern RTICdrDllExport
RTIBool RTICdrTypeObject_copyContiguousBufferTypeObject(
    struct RTICdrTypeObject *dst, const struct RTICdrTypeObject *src);

extern RTICdrDllExport
unsigned int RTICdrTypeObject_getSerializedSampleSize(
    void * endpoint_data,
    RTIBool include_encapsulation,
    RTIEncapsulationId encapsulation_id,
    unsigned int current_alignment,
    const struct RTICdrTypeObject * sample);

extern RTICdrDllExport RTIBool RTICdrTypeObject_getFullyQualifiedName(
        struct RTICdrTypeObject *self,
        char *name,
        size_t maxNameLength);

extern RTICdrDllExport
void RTICdrTypeObject_setTranslateUnboundedToZero(RTIBool value);


#ifdef __cplusplus
    }	/* extern "C" */
#endif

#endif /* cdr_typeObject_h */
