/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)dds_c_asyncwaitset.h    generated by: makeheader    Sat Mar  9 09:39:06 2024
 *
 *		built from:	asyncwaitset.ifc
 */

#ifndef dds_c_asyncwaitset_h
#define dds_c_asyncwaitset_h



#ifndef dds_c_infrastructure_h
  #include "dds_c/dds_c_infrastructure.h"
#endif

#ifndef dds_c_subscription_h
  #include "dds_c/dds_c_subscription.h"
#endif

#ifdef __cplusplus
    extern "C" {
#endif
        


typedef struct DDS_AsyncWaitSetImpl DDS_AsyncWaitSet;


typedef struct DDS_AsyncWaitSetCompletionTokenImpl
        DDS_AsyncWaitSetCompletionToken;


DDSC_VARIABLE_LENGTH_VALUE_TYPE_SUPPORT(
        DDS_AsyncWaitSetProperty_t);

extern DDSCDllVariable 
DDS_AsyncWaitSetCompletionToken *const 
        DDS_ASYNC_WAITSET_COMPLETION_TOKEN_USE_IMPLICIT_AND_WAIT;

extern DDSCDllVariable
DDS_AsyncWaitSetCompletionToken *const 
        DDS_ASYNC_WAITSET_COMPLETION_TOKEN_IGNORE;

extern DDSCDllExport 
DDS_ReturnCode_t DDS_AsyncWaitSetCompletionToken_wait(
        DDS_AsyncWaitSetCompletionToken *self,
        const struct DDS_Duration_t *max_wait);

 
struct DDS_AsyncWaitSetProperty_t {
    /*e \dref_AsyncWaitSetProperty_t_waitset_property */
    struct DDS_WaitSetProperty_t waitset_property;        
    
    /*e \dref_AsyncWaitSetProperty_t_thread_pool_size
     */
    DDS_UnsignedLong thread_pool_size;
    
    /*e \dref_AsyncWaitSetProperty_t_thread_settings
     */
    struct DDS_ThreadSettings_t thread_settings;  
    
    /*e \dref_AsyncWaitSetProperty_t_thread_name_prefix
     */
    char * thread_name_prefix;
    
    /*e \dref_AsyncWaitSetProperty_t_wait_timeout
     */
    struct DDS_Duration_t wait_timeout;
    
    /*e \dref_AsyncWaitSetProperty_t_level
     */
    DDS_Long level;

    DDSCPP_VARIABLE_LENGTH_VALUE_TYPE_SUPPORT(
            DDS_AsyncWaitSetProperty_t)
};

extern DDSCDllVariable 
const struct DDS_AsyncWaitSetProperty_t DDS_ASYNC_WAITSET_PROPERTY_DEFAULT;

extern DDSCDllExport
RTIBool DDS_AsyncWaitSetProperty_is_equal(
        const struct DDS_AsyncWaitSetProperty_t *self,
        const struct DDS_AsyncWaitSetProperty_t *other);


typedef void (*DDS_AsyncWaitSetListener_OnThreadSpawnedCallback)(
        void *listener_data,
        DDS_UnsignedLongLong thread_id);


typedef void (*DDS_AsyncWaitSetListener_OnThreadDeletedCallback)(
        void *listener_data,
        DDS_UnsignedLongLong thread_id);


typedef void (*DDS_AsyncWaitSetListener_OnWaitTimeoutCallback)(
        void *listener_data,
        DDS_UnsignedLongLong thread_id);


typedef void (*DDS_AsyncWaitSetListener_OnConditionAttachedCallback)(
        void *listener_data,
        DDS_Condition *condition);


typedef void (*DDS_AsyncWaitSetListener_OnConditionDetachedCallback)(
        void *listener_data,
        DDS_Condition *condition);


struct DDS_AsyncWaitSetListener {
    /*ce \dref_AsyncWaitSetListener_listener_data
     */
    void *listener_data;

    /*ce \dref_AsyncWaitSetListener_on_thread_spawned
     */
    DDS_AsyncWaitSetListener_OnThreadSpawnedCallback
            on_thread_spawned;

    /*ce \dref_AsyncWaitSetListener_on_thread_deleted
     */
    DDS_AsyncWaitSetListener_OnThreadDeletedCallback
            on_thread_deleted;

    /*ce \dref_AsyncWaitSetListener_on_wait_timeout
     */
    DDS_AsyncWaitSetListener_OnWaitTimeoutCallback
            on_wait_timeout;

    /*ci
     * @brief handles the notification of a condition attachment
     */
    DDS_AsyncWaitSetListener_OnConditionAttachedCallback
            on_condition_attached;
    
    /*ci 
     * @brief handles the notification of a condition detachment
     */
    DDS_AsyncWaitSetListener_OnConditionDetachedCallback
            on_condition_detached;
};


#define DDS_AsyncWaitSetListener_INITIALIZER { \
        NULL, \
        NULL, \
        NULL, \
        NULL, \
        NULL, \
        NULL}

extern DDSCDllExport 
DDS_ReturnCode_t DDS_AsyncWaitSet_get_property(
    DDS_AsyncWaitSet *self,
    struct DDS_AsyncWaitSetProperty_t *property);

extern DDSCDllExport 
DDS_ReturnCode_t DDS_AsyncWaitSet_detach_condition(
        DDS_AsyncWaitSet *self,
        DDS_Condition *condition);

extern DDSCDllExport 
DDS_ReturnCode_t DDS_AsyncWaitSet_detach_condition_with_completion_token(
        DDS_AsyncWaitSet *self,
        DDS_Condition *condition,
        DDS_AsyncWaitSetCompletionToken *completion_token);

extern DDSCDllExport 
DDS_ReturnCode_t DDS_AsyncWaitSet_attach_condition(
        DDS_AsyncWaitSet *self,
        DDS_Condition *condition);

extern DDSCDllExport 
DDS_ReturnCode_t DDS_AsyncWaitSet_attach_condition_with_completion_token(
        DDS_AsyncWaitSet *self,
        DDS_Condition *condition,
        DDS_AsyncWaitSetCompletionToken *completion_token);

extern DDSCDllExport 
DDS_ReturnCode_t DDS_AsyncWaitSet_unlock_condition(
        DDS_AsyncWaitSet *self,
        DDS_Condition *condition);

extern DDSCDllExport 
DDS_ReturnCode_t DDS_AsyncWaitSet_get_conditions(
        DDS_AsyncWaitSet *self,
        struct DDS_ConditionSeq *attached_conditions);

extern DDSCDllExport
DDS_ReturnCode_t DDS_AsyncWaitSet_stop(
        DDS_AsyncWaitSet *self);

extern DDSCDllExport 
DDS_ReturnCode_t DDS_AsyncWaitSet_stop_with_completion_token(
        DDS_AsyncWaitSet *self,
        DDS_AsyncWaitSetCompletionToken *completion_token);

extern DDSCDllExport 
DDS_ReturnCode_t DDS_AsyncWaitSet_start(
        DDS_AsyncWaitSet *self);

extern DDSCDllExport 
DDS_ReturnCode_t DDS_AsyncWaitSet_start_with_completion_token(
        DDS_AsyncWaitSet *self,
        DDS_AsyncWaitSetCompletionToken *completion_token);

extern DDSCDllExport 
DDS_Boolean DDS_AsyncWaitSet_is_started(
        DDS_AsyncWaitSet *self);

extern DDSCDllExport 
DDS_AsyncWaitSetCompletionToken * DDS_AsyncWaitSet_create_completion_token(
        DDS_AsyncWaitSet *self);

extern DDSCDllExport 
DDS_ReturnCode_t DDS_AsyncWaitSet_delete_completion_token(
        DDS_AsyncWaitSet *self,
        DDS_AsyncWaitSetCompletionToken *completion_token);

extern DDSCDllExport 
DDS_AsyncWaitSet * DDS_AsyncWaitSet_new(
        const struct DDS_AsyncWaitSetProperty_t *property);

extern DDSCDllExport 
DDS_AsyncWaitSet * DDS_AsyncWaitSet_new_with_listener(
        const struct DDS_AsyncWaitSetProperty_t *property,
        struct DDS_AsyncWaitSetListener *listener);

extern DDSCDllExport 
DDS_AsyncWaitSet * DDS_AsyncWaitSet_new_with_thread_factory(
        const struct DDS_AsyncWaitSetProperty_t *property,
        struct DDS_AsyncWaitSetListener *listener,
        struct DDS_ThreadFactory *thread_factory);

extern DDSCDllExport  
DDS_ReturnCode_t DDS_AsyncWaitSet_delete(DDS_AsyncWaitSet *self);


typedef struct DDS_DataReaderStatusConditionHandlerImpl 
        DDS_DataReaderStatusConditionHandler;

extern DDSCDllExport
DDS_DataReaderStatusConditionHandler *
DDS_DataReaderStatusConditionHandler_new(
        DDS_DataReader *reader,
        const struct DDS_DataReaderListener *listener,
        DDS_StatusMask listener_mask);

extern DDSCDllExport
DDS_ReturnCode_t
DDS_DataReaderStatusConditionHandler_delete(
        DDS_DataReaderStatusConditionHandler *self);


#define DDS_DataReaderStatusConditionHandler_as_condition_handler(self) \
            ((struct DDS_ConditionHandler*) (self))

#define DDS_DataReaderStatusConditionHandler_narrow(handlerPtr) \
            ((DDS_DataReaderStatusConditionHandler *) (handlerPtr)->handler_data)


#ifdef __cplusplus
    }	/* extern "C" */
#endif

#endif /* dds_c_asyncwaitset_h */
