/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)dl_driver_activation_user.h    generated by: makeheader    Sat Mar  9 09:37:23 2024
 *
 *		built from:	activation_user.ifc
 */

#ifndef dl_driver_activation_user_h
#define dl_driver_activation_user_h



#include "dl_driver/dl_driver_dll.h"

/* Predefined exception codes. Range 0-99 is reserved for future usage */
/**
 * @brief No error.
 */
#define RTI_ACTIVATION_LIBRARY_EXCEPTION_OK 0
/**
 * @brief Generic error.
 */
#define RTI_ACTIVATION_LIBRARY_EXCEPTION_ERROR 1
/**
 * @brief The input buffer is not big enough to hold the activation string.
 */
#define RTI_ACTIVATION_LIBRARY_EXCEPTION_INSUFFICIENT_SPACE 2

/**
 * @brief Maximum size of the exception message (including NULL-terminated
 * character).
 */
#define RTI_ACTIVATION_LIBRARY_EXCEPTION_MESSAGE_MAX_SIZE 512
/**
 * @brief Maximum size of the who-to-contact message (including
 * NULL-terminated character).
 */
#define RTI_ACTIVATION_LIBRARY_CONTACT_INFO_MESSAGE_MAX_SIZE 80

/**
 * @brief Exception object for the activation shared library.
 *
 * @see RTI_ActivationLibrary_get_activation_string
 */
struct RTI_ActivationLibraryException {
    /**
    * @brief Exception code.
    */
    int code;
    /**
    * @brief Exception message.
    */
    char message[RTI_ACTIVATION_LIBRARY_EXCEPTION_MESSAGE_MAX_SIZE];
};

/**
 * @brief An initializer for the exception object.
 *
 * @see RTI_ActivationLibraryException
 */
#define RTI_ActivationLibraryException_INITIALIZER \
    { \
        RTI_ACTIVATION_LIBRARY_EXCEPTION_OK,  /* code */ \
        {'\0'} /* message */ \
    }

/**
 * @brief Get the activation string.
 *
 * @pre buffer can be NULL if *size is 0.
 * @pre size cannot be NULL.
 * @pre exception cannot be NULL.
 *
 * @param buffer Out. Buffer where the activation string will be copied.
 * @param size In/Out. In: Size of the buffer. Out: Size of the activation
 * string. The size includes the null terminator.
 * @param user_data In. User data returned by RTI_ActivationLibrary_initialize().
 * @param exception Out. Exception object. If there is no error, code will be
 * set to RTI_ACTIVATION_LIBRARY_EXCEPTION_OK. If there is an error, code
 * will be set to any of the predefined errors
 * (RTI_ACTIVATION_LIBRARY_EXCEPTION_ERROR or
 * RTI_ACTIVATION_LIBRARY_EXCEPTION_INSUFFICIENT_SPACE) or a user-defined
 * error code. The exception message will contain a description of the error.
 *
 * If the input buffer is not big enough to hold the activation string, the
 * function will return RTI_ACTIVATION_LIBRARY_EXCEPTION_INSUFFICIENT_SPACE
 * and *size will be set to the required size of the activation string.
 *
 * This function can be called in parallel by multiple threads.
 *
 * @return 0 if there is an error, 1 otherwise.
 */
DLDRIVERDllExport
int RTI_ActivationLibrary_get_activation_string(
        char *buffer,
        unsigned int *size,
        void *user_data,
        struct RTI_ActivationLibraryException *exception);

/**
 * @brief Initialize the activation library.
 *
 * This function will be called once after loading the activation shared
 * library.
 *
 * @param user_data Out. User data. The library can initialize *user_data to
 * NULL or to a pointer to a library-defined object.
 * @param error_contact_info Out. Who-to-contact message. The library can use
 * this message to change the displayed who-to-contact message in case of an
 * exception.
 * @param exception Out. Exception object. If there is no error, code will be
 * set to RTI_ACTIVATION_LIBRARY_EXCEPTION_OK. If there is an error, code
 * will be set to any of the predefined errors or a user-defined error code.
 *
 * @return 0 if there is an error, 1 otherwise.
 */
DLDRIVERDllExport
int RTI_ActivationLibrary_initialize(
        void **user_data,
        char error_contact_info[RTI_ACTIVATION_LIBRARY_CONTACT_INFO_MESSAGE_MAX_SIZE],
        struct RTI_ActivationLibraryException *exception);

/**
 * @brief Finalize the activation library.
 *
 * This function will be called once after unloading the activation shared
 * library.
 *
 * @param user_data In. User data.
 * @param exception Out. Exception object. If there is no error, code will be
 * set to RTI_ACTIVATION_LIBRARY_EXCEPTION_OK. If there is an error, code
 * will be set to any of the predefined errors or a user-defined error code.
 *
 * @return 0 if there is an error, 1 otherwise.
 */
DLDRIVERDllExport
int RTI_ActivationLibrary_finalize(
        void *user_data,
        struct RTI_ActivationLibraryException *exception);


#endif /* dl_driver_activation_user_h */
