#ifndef OMG_DDS_RPC_EXCEPTION_
#define OMG_DDS_RPC_EXCEPTION_

/*
 * (c) Copyright, Real-Time Innovations, 2020
 * All rights reserved.
 *
 * No duplications, whole or partial, manual or electronic, may be made
 * without express written permission.  Any such copies, or
 * revisions thereof, must display this notice unaltered.
 * This code contains trade secrets of Real-Time Innovations, Inc.
 */

// IMPORTANT: macros.hpp must be the first RTI header included in every header
// file so that symbols are exported correctly on Windows
#include <dds/core/macros.hpp>

#if !defined(RTI_NO_CXX11_RPC_SUPPORT)

#include <future>
#include <dds/core/Exception.hpp>

namespace dds { namespace rpc {

/**
 * @brief Thrown when a Client calls an operation that doesn't exist in the
 * Service.
 *
 * Inherits also from \p std::logic_error
 *
 */
class OMG_DDS_API RemoteUnknownOperationError : public dds::core::Exception,
                                                public std::logic_error {
public:
    RemoteUnknownOperationError()
            : std::logic_error(
                    "The requested operation doesn't exist in the service")
    { }

    RemoteUnknownOperationError(const RemoteUnknownOperationError& src)
            : std::logic_error(src)
    { }

    RemoteUnknownOperationError& operator=(
            const RemoteUnknownOperationError& src)
    {
        std::logic_error::operator=(src);
        return *this;
    }

    virtual ~RemoteUnknownOperationError() noexcept
    { }

    /**
     * @brief Access the message contained in this RemoteUnknownOperationError
     * exception.
     *
     * @return The message.
     */
    virtual const char *what() const noexcept
    {
        return std::logic_error::what();
    }
};

}}  // namespace dds::rpc

#endif  // RTI_NO_CXX11_RPC_SUPPORT
#endif  // OMG_DDS_RPC_EXCEPTION_
