/* $Id$

 (c) Copyright, Real-Time Innovations, 2022.
 All rights reserved.
 No duplications, whole or partial, manual or electronic, may be made
 without express written permission.  Any such copies, or
 revisions thereof, must display this notice unaltered.
 This code contains trade secrets of Real-Time Innovations, Inc.

*/


#ifndef RTI_TEST_HEAPMONITORINGTESTERCONTAINER_HPP_
#define RTI_TEST_HEAPMONITORINGTESTERCONTAINER_HPP_


#include <dds/domain/DomainParticipant.hpp>
#include "rtitest/Tester.hpp"

namespace rti { namespace test {

// A specialized version of TesterContainer that runs heap monitoring if
// enabled. It always finalizes the global state regardless of whether or not
// heap monitoring is enabled.
class HeapMonitoringTesterContainer : public TesterContainer {
public:
    explicit HeapMonitoringTesterContainer(const std::string &name)
            : TesterContainer(name)
    {
    }

    bool on_tests_begin(const RTITestSetting &setting)
    {
        if (setting.enableHeapMonitoring) {
            RTI_TEST_ASSERT(RTITestSetting_enableHeapMonitoringSupport());
        }
        return true;
    }

    bool on_tests_end(const RTITestSetting &setting)
    {
        // Finalize global state
        dds::domain::DomainParticipant::finalize_participant_factory();
        DDS_TypeCodeFactory_finalize_instance();

        int heap_ok = 1;
        if (setting.enableHeapMonitoring) {
            RTITestSetting_disableHeapMonitoringSupport(&heap_ok);
        }

        return heap_ok != 0;
    }
};

} }

#endif  // RTI_TEST_HEAPMONITORINGTESTERCONTAINER_HPP_

/* End $Id$ */
