/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)ndds_transport_c.h    generated by: makeheader    Sat Mar  9 09:39:09 2024
 *
 *		built from:	ndds_transport_c.ifc
 */

#ifndef ndds_transport_c_h
#define ndds_transport_c_h



  #ifndef transport_interface_h
    #include "transport/transport_interface.h"
  #endif
  #ifndef dds_c_infrastructure_h
    #include "dds_c/dds_c_infrastructure.h"
  #endif  
  #ifndef dds_c_domain_h
    #include "dds_c/dds_c_domain.h"
  #endif

#ifdef __cplusplus
    extern "C" {
#endif

typedef struct {
    DDS_DomainParticipant* _participant;
    struct REDAWeakReference _plugin_handle;
} NDDS_TRANSPORT_HANDLE_TYPE_NATIVE;

typedef NDDS_TRANSPORT_HANDLE_TYPE_NATIVE NDDS_Transport_Handle_t;

 extern DDSCDllVariable const NDDS_Transport_Handle_t
NDDS_TRANSPORT_HANDLE_NIL;

extern DDSCDllExport 
DDS_Boolean NDDS_Transport_Handle_is_nil(const NDDS_Transport_Handle_t* self);


#define NDDS_Transport_Handle_is_nil(self) \
(((void *)self) != NULL && \
memcmp((self),&NDDS_TRANSPORT_HANDLE_NIL,sizeof(NDDS_Transport_Handle_t)) == 0)

extern DDSCDllExport 
NDDS_Transport_Handle_t NDDS_Transport_Support_register_transport(
        DDS_DomainParticipant *participant_in,
        NDDS_Transport_Plugin *transport_in,
        const struct DDS_StringSeq *aliases_in,
        const NDDS_Transport_Address_t *network_address_in);

extern DDSCDllExport 
NDDS_Transport_Handle_t NDDS_Transport_Support_lookup_transport(
        DDS_DomainParticipant *participant_in,
        struct DDS_StringSeq *aliases_out,
        NDDS_Transport_Address_t *network_address_out,
        const NDDS_Transport_Plugin *transport_in);

extern DDSCDllExport 
DDS_ReturnCode_t NDDS_Transport_Support_add_send_route(
        const NDDS_Transport_Handle_t *transport_handle_in,
        const NDDS_Transport_Address_t *address_range_in,
        DDS_Long address_range_bit_count_in);

extern DDSCDllExport 
DDS_ReturnCode_t NDDS_Transport_Support_add_receive_route(
        const NDDS_Transport_Handle_t *transport_handle_in,
        const NDDS_Transport_Address_t *address_range_in,
        DDS_Long address_range_bit_count_in);

extern DDSCDllExport 
DDS_ReturnCode_t NDDS_Transport_Support_get_builtin_transport_property(
        DDS_DomainParticipant *participant_in,
        DDS_TransportBuiltinKind builtin_transport_kind_in,
        struct NDDS_Transport_Property_t *builtin_transport_property_inout);

extern DDSCDllExport 
DDS_ReturnCode_t NDDS_Transport_Support_set_builtin_transport_property(
        DDS_DomainParticipant *participant_in,
        DDS_TransportBuiltinKind builtin_transport_kind_in,
        const struct NDDS_Transport_Property_t* builtin_transport_property_in);

extern DDSCDllExport 
DDS_ReturnCode_t NDDS_Transport_Support_assert_fake_address(
        DDS_DomainParticipant *participant_in,
        NDDS_Transport_Address_t *address,
        NDDS_Transport_ClassId_t transport_class_id);

extern DDSCDllExport 
DDS_ReturnCode_t NDDS_Transport_Support_assert_fake_ipv4_address(
        DDS_DomainParticipant *participant_in,
        NDDS_Transport_Address_t *address);

typedef NDDS_Transport_Plugin* (*NDDS_Transport_create_plugin)(
        NDDS_Transport_Address_t* default_network_address_out,
        const struct DDS_PropertyQosPolicy *property_in);

extern DDSCDllExport
RTIBool NDDS_Transport_Support_parse_interface_list(
        RTI_INT32 *interfaces_list_length,
        char ***interfaces_list,
        int *properties_parsed,
        RTIBool *is_memory_allocated,
        const struct DDS_PropertyQosPolicy *transport_properties,
        const char *property_prefix,
        const char *interface_suffix_property_name,
        const char *interface_list_suffix_property_name);

extern DDSCDllExport
NDDS_Transport_Plugin * NDDS_Transport_UDPv4_create(
        NDDS_Transport_Address_t *default_network_address_out,
        const struct DDS_PropertyQosPolicy *property_in);

extern DDSCDllExport
NDDS_Transport_Plugin * NDDS_Transport_UDPv4_WAN_create(
        NDDS_Transport_Address_t *default_network_address_out,
        const struct DDS_PropertyQosPolicy *property_in);

extern DDSCDllExport
NDDS_Transport_Plugin * NDDS_Transport_UDPv4_create_from_properties_with_prefix(
        NDDS_Transport_Address_t *default_network_address_out,
        const struct DDS_PropertyQosPolicy *property_in,
        const char *propertyPrefix);

extern DDSCDllExport
NDDS_Transport_Plugin * NDDS_Transport_UDPv4_WAN_create_from_properties_with_prefix(
        NDDS_Transport_Address_t *default_network_address_out,
        const struct DDS_PropertyQosPolicy *property_in,
        const char *propertyPrefix);

extern DDSCDllExport 
NDDS_Transport_Plugin* NDDS_Transport_Support_get_transport_plugin(
           DDS_DomainParticipant* participant_in,
           const char * alias_in);

extern DDSCDllExport
DDS_ReturnCode_t NDDS_Transport_Support_get_udpv4_statistics(
        DDS_DomainParticipant* participant_in,
        struct NDDS_Transport_UDPv4_Statistics* statistics);

extern DDSCDllExport
DDS_ReturnCode_t NDDS_Transport_Support_get_udpv6_statistics(
        DDS_DomainParticipant* participant_in,
        struct NDDS_Transport_UDPv6_Statistics* statistics);

extern DDSCDllExport
DDS_ReturnCode_t NDDS_Transport_Support_get_udp_liveliness_statistics(
        DDS_DomainParticipant *participant_in,
        RTI_UINT64 *packets_sent,
        RTI_UINT64 *bytes_sent,
        RTI_UINT64 *packets_received,
        RTI_UINT64 *bytes_received,
        NDDS_Transport_ClassId_t transport_class_id,
        RTIBool is_secure,
        RTIBool is_automatic);

extern DDSCDllExport 
NDDS_Transport_Plugin* NDDS_Transport_UDPv6_create(
        NDDS_Transport_Address_t *default_network_address_out,
        const struct DDS_PropertyQosPolicy *property_in);

extern DDSCDllExport 
NDDS_Transport_Plugin* NDDS_Transport_UDPv6_create_from_properties_with_prefix(
        NDDS_Transport_Address_t *default_network_address_out,
        const struct DDS_PropertyQosPolicy *property_in, 
        const char *propertyPrefix);

extern DDSCDllExport 
NDDS_Transport_Plugin* NDDS_Transport_Shmem_create(
        NDDS_Transport_Address_t *default_network_address_out,
        const struct DDS_PropertyQosPolicy *property_in,
        const struct DDS_ProductVersion_t *minimum_compatibility_version);


#ifdef __cplusplus
    }	/* extern "C" */
#endif

#endif /* ndds_transport_c_h */
