/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)osapi_heap_impl.h    generated by: makeheader    Sat Mar  9 09:37:08 2024
 *
 *		built from:	heap_impl.ifc
 */

#ifndef osapi_heap_impl_h
#define osapi_heap_impl_h


  #include <stdlib.h>

#ifdef __cplusplus
    extern "C" {
#endif

 extern RTIOsapiDllVariable RTIBool RTIOsapiHeap_g_isMonitoringEnabled;


#ifndef RTI_PRECONDITION_TEST
#define RTIOsapiHeap_isMonitoringEnabled() RTIOsapiHeap_g_isMonitoringEnabled
#endif


typedef enum RTIOsapiHeapHeaderGenerationKind {
    RTI_OSAPI_HEAP_HEADER_GENERATION_KIND_DEFAULT,
    RTI_OSAPI_HEAP_HEADER_GENERATION_KIND_FORCE_SKIP,
    RTI_OSAPI_HEAP_HEADER_GENERATION_KIND_FORCE_ADD
} RTIOsapiHeapHeaderGenerationKind;



/*--------------------------------------------------------------------------*/
/* Helper function not to be used directly */
RTIOsapiDllExport RTIBool
RTIOsapiHeap_reallocateMemoryInternal(
        void **ptr,
        size_t size,
        int alignment,
        RTIBool reallocPtr,
        RTIOsapiHeapHeaderGenerationKind heapHeaderGenerationKind,
        const char *caller,
        enum RTIOsapiHeapAllocatorKind allocKind,
        const char *typeName);

/*--------------------------------------------------------------------------*/

/*i \ingroup RTIOsapiHeapClass
 * Used in the RTIOsapiHeap_freeMemoryInternal function when the
 * allocated size is unknown.
 *
 * We should specify the allocated size only in the:
 * RTIOsapiHeap_freeWithoutHeapHeader, because we cannot get the blockSize from
 * the header, because there is no header.
 */
#define RTI_OSAPI_HEAP_UNSPECIFIED_SIZE RTI_SIZE_MAX

/* Helper function not to be used directly */
RTIOsapiDllExport void RTIOsapiHeap_freeMemoryInternal(
        void *ptr,
        RTIOsapiHeapHeaderGenerationKind heapHeaderGenerationKind,
        const char *caller,
        enum RTIOsapiHeapAllocatorKind allocKind,
        size_t size);

/*--------------------------------------------------------------------------*/
/*
 * This function is used by the .NET API. It is required because there is no
 * mechanism to call macros from C# code. This function should not be used by
 * anything other than the new .NET API, instead, use the
 * RTIOsapiHeap_freeStructure macro.
 */
RTIOsapiDllExport void RTIOsapiHeap_freeStructureFunc(void *ptr);

/*--------------------------------------------------------------------------*/
/*
 * This function is used by the .NET API. It is required because there is no
 * mechanism to call macros from C# code. This function should not be used by
 * anything other than the new .NET API, instead, use the
 * RTIOsapiHeap_allocateStructure macro.
 */
RTIOsapiDllExport void *RTIOsapiHeap_allocateStructureFunc(size_t length);

/*--------------------------------------------------------------------------*/
/*
 * This function is used by the .NET API. It is required because there is no
 * mechanism to call macros from C# code. This function should not be used by
 * anything other than the new .NET API, instead, use the
 * RTIOsapiHeap_reallocateString macro.
 */
RTIOsapiDllExport RTIBool RTIOsapiHeap_reallocateStringFunc(
        char **stringStoragePointer,
        size_t stringLength);

/*--------------------------------------------------------------------------*/
/* This needs to stay a MACRO; not for performance reasons but because
   it is the only way to pass a "type" into the "function" 
 */
#define RTIOsapiHeap_allocateStructure(destPointer, destType) \
    RTIOsapiHeap_reallocateMemoryInternal( \
            (void **)(destPointer), \
            sizeof(destType), \
            RTI_OSAPI_ALIGNMENT_DEFAULT, \
            RTI_FALSE, /* reallocPtr */ \
            RTI_OSAPI_HEAP_HEADER_GENERATION_KIND_DEFAULT, \
            "RTIOsapiHeap_allocateStructure", \
            RTI_OSAPI_STRUCT_ALLOC, \
            #destType) \
                ?(*(destPointer) = (destType *)*(destPointer)) \
                :NULL

/*--------------------------------------------------------------------------*/
/* This needs to stay a MACRO; not for performance reasons but because
   it is the only way to pass a "type" into the "function"
 */
#define RTIOsapiHeap_allocateDerivedStructure(                                \
        destBaseDoublePointer__,                                              \
        destBaseType__,                                                       \
        destDerivedType__)                                                    \
    {                                                                         \
        destDerivedType__ *destDerivedPtr__ = NULL;                           \
        RTIOsapiHeap_allocateStructure(&destDerivedPtr__, destDerivedType__); \
        *(destBaseDoublePointer__) = (destBaseType__ *) (destDerivedPtr__);   \
    }

/*--------------------------------------------------------------------------*/
/* This needs to stay a MACRO; not for performance reasons but because
   it is the only way to pass a "type" into the "function" 
 */
#define RTIOsapiHeap_allocateArray(destPointer, elementCount, elementType) \
    RTIOsapiHeap_reallocateMemoryInternal( \
            (void **)(destPointer), \
            ((size_t) (elementCount)) * sizeof(elementType), \
            RTI_OSAPI_ALIGNMENT_DEFAULT, \
            RTI_FALSE, /* reallocPtr */ \
            RTI_OSAPI_HEAP_HEADER_GENERATION_KIND_DEFAULT, \
            "RTIOsapiHeap_allocateArray", \
            RTI_OSAPI_ARRAY_ALLOC, \
            #elementType) \
                ?(*(destPointer) = (elementType *)*(destPointer)) \
                :NULL

/*--------------------------------------------------------------------------*/
#define RTIOsapiHeap_allocateString(stringStoragePointer, stringSize) \
     RTIOsapiHeap_reallocateMemoryInternal( \
             (void **)(stringStoragePointer), \
             (size_t)(stringSize)+1, \
             RTI_OSAPI_ALIGNMENT_DEFAULT, \
             RTI_FALSE, /* reallocPtr */ \
             RTI_OSAPI_HEAP_HEADER_GENERATION_KIND_DEFAULT, \
             "RTIOsapiHeap_allocateString", \
             RTI_OSAPI_STRING_ALLOC, \
             "char") \
                 ?(*(stringStoragePointer) = (char *)*(stringStoragePointer)) \
                 :NULL

/*--------------------------------------------------------------------------*/
#define RTIOsapiHeap_allocateBufferNotAligned(buffer__, size__) \
    RTIOsapiHeap_reallocateMemoryInternal(                      \
            (void **) (buffer__),                               \
            (size_t) (size__),                                  \
            RTI_OSAPI_ALIGNMENT_DEFAULT,                        \
            RTI_FALSE, /* reallocPtr */                         \
            RTI_OSAPI_HEAP_HEADER_GENERATION_KIND_DEFAULT,      \
            "RTIOsapiHeap_allocateBufferNotAligned",            \
            RTI_OSAPI_BUFFER_ALLOC,                             \
            "unsigned char")                                    \
            ? (*(buffer__) = *(buffer__))                       \
            : NULL

/*--------------------------------------------------------------------------*/
#define RTIOsapiHeap_allocateBufferNotAlignedWithAllocKind(       \
        buffer__,                                                 \
        size__,                                                   \
        allocKind__)                                              \
    RTIOsapiHeap_reallocateMemoryInternal(                        \
            (void **) (buffer__),                                 \
            (size_t) (size__),                                    \
            RTI_OSAPI_ALIGNMENT_DEFAULT,                          \
            RTI_FALSE, /* reallocPtr */                           \
            RTI_OSAPI_HEAP_HEADER_GENERATION_KIND_DEFAULT,        \
            "RTIOsapiHeap_allocateBufferNotAlignedWithAllocKind", \
            (allocKind__),                                        \
            "unsigned char")                                      \
            ? (*(buffer__) = *(buffer__))                         \
            : NULL

/*--------------------------------------------------------------------------*/
#define RTIOsapiHeap_allocateBufferAlignedWithTypeName(              \
        buffer__,                                                    \
        size__,                                                      \
        alignment__,                                                 \
        typeName__)                                                  \
    RTIOsapiHeap_reallocateMemoryInternal(                           \
            (void **) (buffer__),                                    \
            (size_t) (size__),                                       \
            (alignment__),                                           \
            RTI_FALSE, /* reallocPtr */                              \
            RTI_OSAPI_HEAP_HEADER_GENERATION_KIND_FORCE_ADD,         \
            "RTIOsapiHeap_allocateBufferAligned",                    \
            RTI_OSAPI_BUFFER_ALIGN_ALLOC,                            \
            ((const void *) (typeName__)) != NULL ? (typeName__)     \
                                                  : "unsigned char") \
            ? (*(buffer__) = *(buffer__))                            \
            : NULL

#define RTIOsapiHeap_allocateBufferAligned(buffer, size, alignment) \
    RTIOsapiHeap_allocateBufferAlignedWithTypeName( \
            buffer, \
            size, \
            alignment, \
            "unsigned char")

/*--------------------------------------------------------------------------*/
#define RTIOsapiHeap_allocateBuffer RTIOsapiHeap_allocateBufferAligned

/*--------------------------------------------------------------------------*/
#define RTIOsapiHeap_freeStructure(dest) \
    RTIOsapiHeap_freeMemoryInternal( \
            (void *) (dest), \
            RTI_OSAPI_HEAP_HEADER_GENERATION_KIND_DEFAULT, \
            "RTIOsapiHeap_freeStructure", \
            RTI_OSAPI_STRUCT_ALLOC, \
            RTI_OSAPI_HEAP_UNSPECIFIED_SIZE)

/*--------------------------------------------------------------------------*/
#define RTIOsapiHeap_freeArray(dest) \
    RTIOsapiHeap_freeMemoryInternal( \
            (void *) (dest), \
            RTI_OSAPI_HEAP_HEADER_GENERATION_KIND_DEFAULT, \
            "RTIOsapiHeap_freeArray", \
            RTI_OSAPI_ARRAY_ALLOC, \
            RTI_OSAPI_HEAP_UNSPECIFIED_SIZE)

/*--------------------------------------------------------------------------*/
#define RTIOsapiHeap_freeString(stringStorage) \
    RTIOsapiHeap_freeMemoryInternal( \
            (void *) (stringStorage), \
            RTI_OSAPI_HEAP_HEADER_GENERATION_KIND_DEFAULT, \
            "RTIOsapiHeap_freeString", \
            RTI_OSAPI_STRING_ALLOC, \
            RTI_OSAPI_HEAP_UNSPECIFIED_SIZE)

/*--------------------------------------------------------------------------*/
#define RTIOsapiHeap_freeBuffer(buffer) \
    RTIOsapiHeap_freeMemoryInternal( \
            (void *) (buffer), \
            RTI_OSAPI_HEAP_HEADER_GENERATION_KIND_FORCE_ADD, \
            "RTIOsapiHeap_freeBufferAligned", \
            RTI_OSAPI_BUFFER_ALIGN_ALLOC, \
            RTI_OSAPI_HEAP_UNSPECIFIED_SIZE)

/*--------------------------------------------------------------------------*/
#define RTIOsapiHeap_freeBufferNotAligned(buffer) \
    RTIOsapiHeap_freeMemoryInternal( \
            (void *) (buffer), \
            RTI_OSAPI_HEAP_HEADER_GENERATION_KIND_DEFAULT, \
            "RTIOsapiHeap_freeBufferNotAligned", \
            RTI_OSAPI_BUFFER_ALLOC, \
            RTI_OSAPI_HEAP_UNSPECIFIED_SIZE)

/*--------------------------------------------------------------------------*/
#define RTIOsapiHeap_freeBufferAligned RTIOsapiHeap_freeBuffer

/*--------------------------------------------------------------------------*/
#define RTIOsapiHeap_reallocateArray(destPointer, elementCount, elementType) \
     RTIOsapiHeap_reallocateMemoryInternal( \
             (void **)(destPointer), \
             ((size_t) (elementCount)) * sizeof(elementType), \
             RTI_OSAPI_ALIGNMENT_DEFAULT, \
             RTI_TRUE, /* reallocPtr */ \
             RTI_OSAPI_HEAP_HEADER_GENERATION_KIND_DEFAULT, \
             "RTIOsapiHeap_reallocateArray", \
             RTI_OSAPI_ARRAY_ALLOC, \
             #elementType) \
                 ?(*(destPointer) = (elementType *)*(destPointer)), RTI_TRUE \
                 :RTI_FALSE

/*--------------------------------------------------------------------------*/
#define RTIOsapiHeap_reallocateString(stringStoragePointer__, stringLength__) \
    (RTIOsapiHeap_reallocateMemoryInternal( \
            (void **) (stringStoragePointer__), \
            (size_t) ((stringLength__) + 1), \
            RTI_OSAPI_ALIGNMENT_DEFAULT, \
            RTI_TRUE, /* reallocPtr */ \
            RTI_OSAPI_HEAP_HEADER_GENERATION_KIND_DEFAULT, \
            "RTIOsapiHeap_reallocateString",\
            RTI_OSAPI_STRING_ALLOC, \
            "char") \
            ? (*(stringStoragePointer__) = (char *) *(stringStoragePointer__)), \
              RTI_TRUE \
            : RTI_FALSE)

/*--------------------------------------------------------------------------*/
/* Default-aligned memory should not force a header */
#define RTIOsapiHeap_reallocateBufferNotAligned(buffer__, size__) \
    RTIOsapiHeap_reallocateMemoryInternal(                        \
            (void **) (buffer__),                                 \
            (size_t) (size__),                                    \
            RTI_OSAPI_ALIGNMENT_DEFAULT,                          \
            RTI_TRUE, /* reallocPtr */                            \
            RTI_OSAPI_HEAP_HEADER_GENERATION_KIND_DEFAULT,        \
            "RTIOsapiHeap_reallocateBufferNotAligned",            \
            RTI_OSAPI_BUFFER_ALLOC,                               \
            "unsigned char")                                      \
            ? (*(buffer__) = *(buffer__)),                        \
            RTI_TRUE : RTI_FALSE

/*--------------------------------------------------------------------------*/
/* Aligned memory always requires a header */
#define RTIOsapiHeap_reallocateBufferAligned(buffer__, size__, alignment__) \
    RTIOsapiHeap_reallocateMemoryInternal(                                  \
            (void **) (buffer__),                                           \
            (size_t) (size__),                                              \
            (alignment__),                                                  \
            RTI_TRUE, /* reallocPtr */                                      \
            RTI_OSAPI_HEAP_HEADER_GENERATION_KIND_FORCE_ADD,                \
            "RTIOsapiHeap_reallocateBufferAligned",                         \
            RTI_OSAPI_BUFFER_ALIGN_ALLOC,                                   \
            "unsigned char ")                                               \
            ? (*(buffer__) = *(buffer__)),                                  \
            RTI_TRUE : RTI_FALSE

/*--------------------------------------------------------------------------*/
#define RTIOsapiHeap_realloc(ptr, size) \
    RTIOsapiHeap_reallocateMemoryInternal( \
            (void **)&(ptr), \
            (size_t)(size), \
            RTI_OSAPI_ALIGNMENT_DEFAULT, \
            RTI_TRUE, /* reallocPtr */ \
            RTI_OSAPI_HEAP_HEADER_GENERATION_KIND_DEFAULT, \
            "RTIOsapiHeap_realloc", \
            RTI_OSAPI_MALLOC_ALLOC, \
            "unsigned char") ? (ptr) : NULL

/*--------------------------------------------------------------------------*/
#define RTIOsapiHeap_free(ptr) \
    RTIOsapiHeap_freeMemoryInternal( \
            (void **) (ptr), \
            RTI_OSAPI_HEAP_HEADER_GENERATION_KIND_DEFAULT, \
            "RTIOsapiHeap_free", \
            RTI_OSAPI_MALLOC_ALLOC, \
            RTI_OSAPI_HEAP_UNSPECIFIED_SIZE)

#define RTIOsapiHeap_freeWithoutHeapHeader(ptr__, size__) \
    RTIOsapiHeap_freeMemoryInternal( \
            (void **) (ptr__), \
            RTI_OSAPI_HEAP_HEADER_GENERATION_KIND_FORCE_SKIP, \
            "RTIOsapiHeap_free", \
            RTI_OSAPI_MALLOC_ALLOC, \
            (size__))

/*--------------------------------------------------------------------------*/

#define RTIOsapiHeapMonitoringProperty_initialize(__format) \
    (__format)->outputFormat = RTI_OSAPI_HEAP_SNAPSHOT_OUTPUT_FORMAT_STANDARD; \
    (__format)->contentFormat = RTI_OSAPI_HEAP_SNAPSHOT_CONTENT_MASK_DEFAULT

#define RTIOsapiHeapMonitoringProperty_copy(__to, __src) \
    (__to)->outputFormat = (__src)->outputFormat; \
    (__to)->contentFormat = (__src)->contentFormat


extern RTIOsapiDllExport
void RTIOsapiHeapMemoryPerThread_getCurrent(RTI_INT32 *currentMemory);


#ifdef __cplusplus
    }	/* extern "C" */
#endif

#endif /* osapi_heap_impl_h */
