/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)osapi_ntptime_impl.h    generated by: makeheader    Sat Mar  9 09:37:09 2024
 *
 *		built from:	ntptime_impl.ifc
 */

#ifndef osapi_ntptime_impl_h
#define osapi_ntptime_impl_h



#include <stdio.h> /* for sprintf() */

#ifdef __cplusplus
    extern "C" {
#endif



#define RTINtpTime_setZero(time) \
{ \
    (time)->sec = 0; \
    (time)->frac = 0; \
}

#define RTINtpTime_setMax(time) \
{ \
    (time)->sec = RTI_NTP_TIME_SEC_MAX; \
    (time)->frac = RTI_NTP_TIME_FRAC_MAX; \
}

#define RTINtpTime_toDouble(t__, double__) \
{ \
    RTIRtpsTime rtpsTime__; \
    RTIRtpsTime_fromNtptime(&(rtpsTime__), (t__)); \
    (*double__) = (((double) ((rtpsTime__).frac) / 4294967296.0) \
            + (double) (rtpsTime__).sec); \
}

#define RTINtpTime_toUInt64(t__, uint64__) \
{ \
    RTIRtpsTime rtpsTime__; \
    RTIRtpsTime_fromNtptime(&(rtpsTime__), (t__)); \
    (*uint64__) = ((((RTI_UINT64) (rtpsTime__).sec) << 32) | ((RTI_UINT64) (rtpsTime__).frac)); \
}

#define RTINtpTime_fromUInt64(t__, uint64__) \
{ \
    /* \ 
     * Casting of uint64__ to RTI_UINT64 is for safety. Otherwise if for some \
     * reason we pass an RTI_INT64 the shift operation will extend the sign. \
     */ \
    (t__)->sec = (RTI_INT64) (((RTI_UINT64) (uint64__)) >> 32); \
    (t__)->frac = ((RTI_UINT32) (uint64__) & 0xFFFFFFFF); \
} \

#define RTINtpTime_copy(out, in)    \
        (out)->sec = (in)->sec;     \
        (out)->frac = (in)->frac;

#define RTINtpTime_packFromMillisec(time__, s__, msec__) \
  { \
    RTI_UINT32 RTINtpTime_temp = (RTI_UINT32) (msec__); \
    (time__).sec = (RTI_INT64) (s__); \
    (time__).frac = (RTINtpTime_temp << 22) + ((RTINtpTime_temp * 393) << 8); \
  }

#define RTI_TIME_MSEC_IN_SEC 1000
#define RTI_TIME_USEC_IN_SEC 1000000
#define RTI_TIME_NSEC_IN_SEC 1000000000

#define RTINtpTime_unpackToMillisec(s__, msec__, time__) \
  { \
    (s__) = (time__).sec; \
    (msec__) = ((time__).frac - ((time__).frac >> 6) - ((time__).frac >> 7) \
                       + (1 << 21)) \
            >> 22; \
    if (((msec__) >= RTI_TIME_MSEC_IN_SEC) && ((s__) < RTI_NTP_TIME_SEC_MAX)) { \
      (msec__) -= RTI_TIME_MSEC_IN_SEC; \
      (s__)++; \
    } \
  }

#define RTINtpTime_packFromMicrosec(time__, s__, usec__) \
  { \
    RTI_UINT32 RTINtpTime_temp = (RTI_UINT32) (usec__); \
    (time__).sec = (RTI_INT64) (s__); \
    (time__).frac = (RTINtpTime_temp << 12) + ((RTINtpTime_temp * 99) << 1) \
            + ((RTINtpTime_temp * 15 + ((RTINtpTime_temp * 61) >> 7)) >> 4); \
  }

#define RTINtpTime_unpackToMicrosec(s, usec, time) \
{ \
    RTI_UINT32 RTINtpTime_temp = (time).frac; \
    s    = (time).sec; \
    usec = ((time).frac- \
	    (RTINtpTime_temp>>5)- \
	    (RTINtpTime_temp>>7)- \
	    (RTINtpTime_temp>>8)- \
            (RTINtpTime_temp>>9)- \
	    (RTINtpTime_temp>>10)- \
	    (RTINtpTime_temp>>12)- \
            (RTINtpTime_temp>>13)- \
	    (RTINtpTime_temp>>14) + (1<<11)) >> 12; \
    if( ((usec) >= RTI_TIME_USEC_IN_SEC) && ((s) < RTI_NTP_TIME_SEC_MAX) ) { \
        (usec) -= RTI_TIME_USEC_IN_SEC; \
        (s)++; \
    } \
}

#define RTINtpTime_packFromNanosec(time, s, nsec) \
{ \
    (time).sec  = (RTI_INT64) (s); \
    (time).frac = (RTI_UINT32) (((RTI_UINT64)(nsec) * 2305843009ull) >> 29); \
}

#define RTINtpTime_unpackToNanosec(s, nsec, time) \
{ \
    (s) = (time).sec; \
    (nsec) = (RTI_UINT32) ((((RTI_UINT64)(time).frac << 29)/2305843009ull) + 1); \
    if ((time).frac == 0 || (time).frac == 2305843009ul) (nsec)--; \
    if( ((nsec)>=RTI_TIME_NSEC_IN_SEC) && ((s) < RTI_NTP_TIME_SEC_MAX) ) { \
       (nsec) -= RTI_TIME_NSEC_IN_SEC; \
       (s)++; \
    } \
}
    
/*e \ingroup RTINtpTimeClass

  @brief Precision used in bit manipulation for RTINtpTime fraction
         conversions

  The larger the precision, the more bits you pick up in the long
  division to get the fraction.  If precision is too large so that the
  denominator shifted by the precision bits blows the 2^32 limit, the
  result will be incorrect.  For example, if denominator is 60 (a la
  Vx sysClkRate), precision_bits larger than 27 blows the limit and
  you get incorrect result.  In general, the
  maximum precision_bits = 32 - (int)log2(denominator_per_sec),

  where log2(x) can be calculated by:

  \verbatim
    int log2answer = 0;
    while (x >>= 1) {1
        log2answer++;
    }
   \endverbatim

   For 60 (sysClkRateGet()), this would be 27.
   For gettimeofday() with microsec resolution, this would be 12.

   But in practice, that kind of precision is not necessary nor desirable
   since the conversion would be slower.  Instead, the precision is
   only important relative to the denominator.  If your clock frequency
   is low (eg 60 Hz), you only have 16.6 ms resolution anyway.  If you are
   within a tenth of your clock resolution that is probably good enough.

   If we generalized too much and tried to handle nanosec resolution "ticks"
   we couldn't guarantee adequate acuracy for all ranges.  So intenally, we
   just pick a reasonable number that gives us adequate accuracy over large
   ranges.  I wanted a precision that will work well up to 1e6 tick rate
   (thinking about UNIX gettimeofday()), and 7 passed the test.


*/
#define RTI_NTP_TIME_PRECISION_BITS_DEFAULT 7

#define RTINtpTime_packFromFraction(time, numerator, denominator_per_sec) \
    RTINtpTime_packFromFractionPrecise((time), \
                                       (numerator), (denominator_per_sec), \
                                       RTI_NTP_TIME_PRECISION_BITS_DEFAULT)

/*e \ingroup RTINtpTimeClass

  @brief Windows-only macro to convert FILETIME into the ntp time.

  @param time RTINtpTime to contain the result.  The existing
  content of time does not affect the result.

  @param fileTime Contains a 64-bit value representing the number of
  100-nanosecond intervals since January 1, 1601 (UTC).
 */
#ifdef RTI_WIN32
    #define RTINtpTime_packFromFILETIME(time__, fileTime__) \
{ \
    RTI_UINT64 timeUsec__; \
    timeUsec__ = (fileTime__).dwHighDateTime; \
    timeUsec__ <<= 32;  \
    timeUsec__ |= ((fileTime__).dwLowDateTime / 10); \
    RTINtpTime_packFromMicrosec( \
            (time__), \
            (timeUsec__ / RTI_TIME_USEC_IN_SEC), \
            (RTI_UINT32) (timeUsec__ % RTI_TIME_USEC_IN_SEC)); \
}
#endif

#define RTINtpTime_packFromFractionPrecise(time, \
                                           numerator, denominator_per_sec, \
                                           precisionBits) \
{ \
    RTI_UINT32 RTINtpTime_current_bit = 0x80000000; \
    RTI_UINT32 RTINtpTime_current_units = (denominator_per_sec)<<precisionBits; \
    RTI_UINT32 RTINtpTime_precision_numerator = \
        (((RTI_UINT32) numerator)%(denominator_per_sec))<< precisionBits; \
    (time).sec  = (RTI_INT64) (numerator)/(denominator_per_sec); \
    (time).frac = 0; \
    while (RTINtpTime_current_units >>= 1) { \
        (time).frac += \
            (((RTINtpTime_precision_numerator) >= RTINtpTime_current_units) ? \
             (((RTINtpTime_precision_numerator) -= RTINtpTime_current_units), RTINtpTime_current_bit) : 0); \
        RTINtpTime_current_bit >>= 1; \
    } \
}

#define RTINtpTime_unpackToFraction(numerator, denominator_per_sec, time) \
    RTINtpTime_unpackToFractionPrecise((numerator), \
                                       (denominator_per_sec), (time), \
                                       RTI_NTP_TIME_PRECISION_BITS_DEFAULT)

#define RTINtpTime_unpackToFractionPrecise(numerator, \
                                           denominator_per_sec, time, \
                                           precisionBits) \
{ \
    RTI_UINT32 RTINtpTime_current_bit = 0x80000000; \
    RTI_UINT32 RTINtpTime_current_units = (denominator_per_sec)<<(precisionBits);\
    RTI_UINT32 RTINtpTime_precision_numerator = 0; \
    if (RTI_NTP_TIME_SEC_MAX / denominator_per_sec < (time).sec) { \
        numerator = (RTI_UINT32) RTI_NTP_TIME_SEC_MAX; \
    } else { \
        numerator = (denominator_per_sec) * (RTI_UINT32) (time).sec; \
        while ((RTINtpTime_current_units >>= 1)) { \
            RTINtpTime_precision_numerator += ((time).frac & RTINtpTime_current_bit) ? RTINtpTime_current_units:0;\
            RTINtpTime_current_bit >>= 1; \
        } \
        numerator += (RTINtpTime_precision_numerator + \
                      (1<<((precisionBits)-1)))>>(precisionBits);\
    } \
}


#define RTINtpTime_compare(time1, time2) \
((((time1).sec) > ((time2).sec)) ? 1 : \
 ((((time1).sec) < ((time2).sec)) ? -1 : \
  ((((time1).frac) > ((time2).frac)) ? 1 : \
   ((((time1).frac) < ((time2).frac)) ? -1 : 0))))


#define RTINtpTime_min(answer, time1, time2) \
{\
    (answer) = \
        ((((time1).sec) < ((time2).sec)) ? (time1) : \
         ((((time1).sec) > ((time2).sec)) ? (time2) : \
          ((((time1).frac) < ((time2).frac)) ? (time1) : (time2) ))); \
}

#define RTINtpTime_max(answer, time1, time2) \
{\
    (answer) = \
        ((((time1).sec) > ((time2).sec)) ? (time1) : \
         ((((time1).sec) < ((time2).sec)) ? (time2) : \
          ((((time1).frac) > ((time2).frac)) ? (time1) : (time2) ))); \
}


/*
 * This calculation is made in such a way that we avoid any overflow
 * operation.
 * This operation: (t1).frac - ((RTI_NTP_TIME_FRAC_MAX - (t2).frac) + 1);
 * is similar to: t1.frac + t2.frac
 * but without any overflow.
 * The +1 operator is there because the (RTI_NTP_TIME_FRAC_MAX + 1) it's the
 * same as 1 second, and that's the real maximum.
 */
#define RTINtpTime_add(answer, t1, t2) \
  { \
    if (((t1).sec >= RTI_NTP_TIME_SEC_MAX) \
            || ((t2).sec >= RTI_NTP_TIME_SEC_MAX)) { \
      (answer).sec = RTI_NTP_TIME_SEC_MAX; \
      (answer).frac = RTI_NTP_TIME_FRAC_MAX; \
    } else { \
      RTI_INT64 resSec = (t1).sec + (t2).sec; \
      if (resSec > RTI_NTP_TIME_SEC_MAX) { \
        (answer).sec = RTI_NTP_TIME_SEC_MAX; \
      } else if (resSec < RTI_NTP_TIME_SEC_MIN) { \
        (answer).sec = RTI_NTP_TIME_SEC_MIN; \
      } else { \
        (answer).sec = resSec; \
      } \
      if ((RTI_NTP_TIME_FRAC_MAX - (t1).frac) < (t2).frac) { \
        (answer).frac = (t1).frac - ((RTI_NTP_TIME_FRAC_MAX - (t2).frac) + 1); \
        if ((answer).sec < RTI_NTP_TIME_SEC_MAX) { \
          (answer).sec++; \
        } else { \
          (answer).frac = RTI_NTP_TIME_FRAC_MAX; \
        } \
      } else { \
        (answer).frac = (t1).frac + (t2).frac; \
      } \
    } \
  }

/*
 * This calculation is made in such a way that we avoid any overflow
 * operation.
 * This operation: (RTI_NTP_TIME_FRAC_MAX - (t2).frac) + (t1).frac + 1; \
 * is similar to: t1.frac - t2.frac
 * but without any overflow.
 * The +1 operator is there because the (RTI_NTP_TIME_FRAC_MAX + 1) it's the
 * same as 1 second, and that's the real maximum.
 */
#define RTINtpTime_subtract(answer, t1, t2) \
{ \
    RTI_INT64 resSec = (t1).sec - (t2).sec; \
    if (resSec > RTI_NTP_TIME_SEC_MAX) { \
        (answer).sec = RTI_NTP_TIME_SEC_MAX; \
    } else if (resSec < RTI_NTP_TIME_SEC_MIN) { \
        (answer).sec = RTI_NTP_TIME_SEC_MIN; \
    } else { \
        (answer).sec = resSec; \
    } \
    if ((t2).frac > (t1).frac) { \
        (answer).frac = (RTI_NTP_TIME_FRAC_MAX - (t2).frac) + (t1).frac + 1; \
        if ((answer).sec > RTI_NTP_TIME_SEC_MIN) { \
            (answer).sec--; \
        } else { \
            (answer).frac = 0; \
        } \
    } else { \
        (answer).frac = (t1).frac - (t2).frac; \
    } \
}

#define RTINtpTime_shiftLeft(answer, time, shift) \
{ \
    if (shift) { \
        RTI_UINT64 RTINtpTime_mask = 0xffffffff; \
        (answer).sec = ((time).sec)<<shift; \
        RTINtpTime_mask = RTINtpTime_mask<<(32 - shift); \
        RTINtpTime_mask &= (time).frac; \
        RTINtpTime_mask = RTINtpTime_mask>>(32 - shift); \
        (answer).sec |= (RTI_INT64) RTINtpTime_mask; \
        (answer).frac = ((time).frac)<<shift; \
    } else { \
        answer = time; \
    } \
}

#define RTINtpTime_shiftRight(answer, time, shift) \
{ \
    if (shift) { \
        RTI_UINT64 mask__ = 0xffffffffffffffffULL; \
        RTI_UINT32 smallMask__ = 0xffffffffU; \
        (answer).frac = ((time).frac)>>shift; \
        mask__ = mask__>>(64 - shift); \
        mask__ &= (RTI_UINT64) (time).sec; \
        smallMask__ = (RTI_UINT32) mask__; \
        smallMask__ = smallMask__<<(32 - shift); \
        (answer).frac |= smallMask__; \
        (answer).sec = ((time).sec)>>shift; \
    } else { \
        answer = time; \
    } \
}

#define RTINtpTime_increment(answer, time) \
        RTINtpTime_add(answer, answer, time);

#define RTINtpTime_decrement(answer, time) \
        RTINtpTime_subtract(answer, answer, time);

#define RTINtpTime_incrementInfinitesimally(time) \
    ((++((time).frac)) == 0) ? ++(time).sec : 0

#define RTINtpTime_isInfinite(time) (((void *) (time) == NULL) || \
                                    ((time)->sec >= RTI_NTP_TIME_SEC_MAX))
#define RTINtpTime_isAuto(time) (((void *) (time) != NULL) && \
                                (((time)->sec == RTI_NTP_TIME_SEC_AUTO) && \
                                ((time)->frac == RTI_NTP_TIME_FRAC_AUTO)))
#define RTINtpTime_isInvalid(time) (((void *) (time) != NULL) && \
                                (((time)->sec == RTI_NTP_TIME_SEC_INVALID) && \
                                ((time)->frac == RTI_NTP_TIME_FRAC_INVALID)))
#define RTINtpTime_compareToZero(time) \
(((const void*)time == NULL) /* infinite > 0 */                       \
 ? 1                                                                  \
 : ((time)->sec > 0 /* positive > 0 */                                \
    ? (time)->sec                                                     \
    : (((time)->sec == 0) /* if sec == 0, */                          \
       ? ((time)->frac != 0) /* if frac == 0, return 0, otherwise 1 */\
       : -1))) /* else sec < 0 => less than 0 */

#define RTINtpTime_toString(me, str) \
    (sprintf(str, "{%08llx,%08X}", (me)->sec, (me)->frac), (str))


#define RTINtpTime_scale(answer, time, percentScaler) \
{ \
    RTI_UINT32 prodHigh, prodLow, normalizedScaler, p3, p2, p1, p0; \
    const RTI_UINT32 LOW_MASK = 0x0000FFFF; \
    prodLow = (time)->frac; \
    prodHigh = (RTI_UINT32)((time)->sec); \
    normalizedScaler = (RTI_UINT32)(percentScaler); \
    p0 = (normalizedScaler) * (prodLow & LOW_MASK); \
    p1 = (normalizedScaler) * (prodLow >> 16) + (p0 >> 16); \
    p2 = (normalizedScaler) * (prodHigh & LOW_MASK) + (p1 >> 16); \
    p3 = (normalizedScaler) * (prodHigh >> 16) + (p2 >> 16); \
    prodLow = (p1 << 16) | (p0 & LOW_MASK); \
    prodHigh = (p3 << 16) | (p2 & LOW_MASK); \
    /* 655/2^16 ~= 1/100 */ \
    p0 = (655) * (prodLow & LOW_MASK); \
    p1 = (655) * (prodLow >> 16) + (p0 >> 16); \
    p2 = (655) * (prodHigh & LOW_MASK) + (p1 >> 16); \
    p3 = (655) * (prodHigh >> 16) + (p2 >> 16); \
    (answer)->sec = (RTI_INT64) p3; \
    (answer)->frac =  (p2 << 16) | (p1 & LOW_MASK); \
}

#define RTINtpTime_divide(answer, time, divisor) \
{ \
    RTI_UINT64 tempAns__; \
    RTI_UINT32 smallSec__ = (RTI_UINT32) (time)->sec; \
    tempAns__ = ((smallSec__) << 16) | ((time)->frac) >> 16; \
    tempAns__ /= (RTI_UINT64) divisor; \
    (answer)->sec = (RTI_INT64) (tempAns__ >> 16); \
    (answer)->frac = (RTI_UINT32) tempAns__ << 16; \
}

#ifdef __USE_TIME_BITS64
    #define RTINtpTime_toTimeval(time__, timeval__) \
    { \
        RTI_INT64 sec__; \
        RTI_UINT32 usec__; \
        RTINtpTime_unpackToMicrosec((sec__), (usec__), *(time__)); \
        (timeval__)->tv_sec = sec__; \
        (timeval__)->tv_usec = usec__; \
    }

    #define RTINtpTime_toCalendarTime(time__, calendarTime__) \
    { \
        RTI_INT64 sec__; \
        RTI_UINT32 usec__; \
        RTINtpTime_unpackToMicrosec((sec__), (usec__), *(time__)); \
        *(calendarTime__) = sec__; \
        if (usec__ >= 500000 && *(calendarTime__) < RTI_INT64_MAX) { \
            ++(*(calendarTime__)); \
        } \
    }

    #define RTINtpTime_toTimespec(time__, timespec__) \
    { \
        RTI_INT64 sec__; \
        RTI_UINT32 nsec__; \
        RTINtpTime_unpackToNanosec((sec__), (nsec__), *(time__)); \
        (timespec__)->tv_sec = sec__; \
        (timespec__)->tv_nsec = (long) nsec__; \
    }

#else
    #define RTINtpTime_toTimeval(time__, timeval__) \
    { \
        RTI_INT64 sec__; \
        RTI_UINT32 usec__; \
        RTINtpTime_unpackToMicrosec((sec__), (usec__), *(time__)); \
        if (sec__ > RTI_INT32_MAX) { \
            (timeval__)->tv_sec = RTI_INT32_MAX; \
        } else { \
            (timeval__)->tv_sec = (RTI_INT32) sec__; \
        } \
        (timeval__)->tv_usec = usec__; \
    }

    #define RTINtpTime_toCalendarTime(time__, calendarTime__) \
    { \
        RTI_INT64 sec__; \
        RTI_UINT32 usec__; \
        RTINtpTime_unpackToMicrosec((sec__), (usec__), *(time__)); \
        if (sizeof(*(calendarTime__)) == sizeof(sec__)) { \
            *(calendarTime__) = sec__; \
        } else { \
            if (sec__ > RTI_INT32_MAX) { \
                *(calendarTime__) = RTI_INT32_MAX; \
            } else { \
                *(calendarTime__) = (RTI_INT32) sec__; \
            } \
        } \
        if (usec__ >= 500000 && *(calendarTime__) < RTI_INT32_MAX) { \
            ++(*(calendarTime__)); \
        } \
    }

    #define RTINtpTime_toTimespec(time__, timespec__) \
    { \
        RTI_INT64 sec__; \
        RTI_UINT32 nsec__; \
        RTINtpTime_unpackToNanosec((sec__), (nsec__), *(time__)); \
        if (sec__ > RTI_INT32_MAX) { \
            (timespec__)->tv_sec = RTI_INT32_MAX; \
        } else { \
            (timespec__)->tv_sec = (RTI_INT32) sec__; \
        } \
        (timespec__)->tv_nsec = (long) nsec__; \
    }
#endif

#define RTINtpTime_toBuffer(time__, buffer__) \
    RTIOsapiMemory_copy((buffer__), (time__), RTI_OSAPI_NTPTIME_SERIALIZED_SIZE);



#ifdef __cplusplus
    }	/* extern "C" */
#endif

#endif /* osapi_ntptime_impl_h */
