/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)osapi_platform.h    generated by: makeheader    Sat Mar  9 09:37:09 2024
 *
 *		built from:	platform.ifc
 */

#ifndef osapi_platform_h
#define osapi_platform_h



/*
 * Define CLANG flag for all Darwin archs
 */

#ifdef RTI_DARWIN
  #ifndef RTI_CLANG
    #define RTI_CLANG
  #endif
#endif

/*
 * ENABLE_FACE_COMPLIANCE: These match the definitions in Micro
 * 0 - No compliance with FACE is required
 * 1 - Comply with the SECURITY profile (not supported)
 * 2 - Comply with the SAFETY_BASE profile (not supported)
 * 3 - Comply with the SAFETY_EXTENDED profile (not supported)
 * 4 - Comply with the GENERAL profile (supported)
 */
#ifndef FACE_COMPLIANCE_LEVEL_NONE
#define FACE_COMPLIANCE_LEVEL_NONE             0
#endif
#ifndef FACE_COMPLIANCE_LEVEL_SECURITY
#define FACE_COMPLIANCE_LEVEL_SECURITY         1
#endif
#ifndef FACE_COMPLIANCE_LEVEL_SAFETY_BASE
#define FACE_COMPLIANCE_LEVEL_SAFETY_BASE      2
#endif
#ifndef FACE_COMPLIANCE_LEVEL_SAFETY_EXTENDED
#define FACE_COMPLIANCE_LEVEL_SAFETY_EXTENDED  3
#endif
#ifndef FACE_COMPLIANCE_LEVEL_GENERAL
#define FACE_COMPLIANCE_LEVEL_GENERAL          4
#endif

#ifndef ENABLE_FACE_COMPLIANCE
#define ENABLE_FACE_COMPLIANCE                   FACE_COMPLIANCE_LEVEL_NONE
#endif

#if (ENABLE_FACE_COMPLIANCE != FACE_COMPLIANCE_LEVEL_NONE)
  #if (ENABLE_FACE_COMPLIANCE != FACE_COMPLIANCE_LEVEL_GENERAL)
  #error "RTI Connext DDS only supports FACE GENERAL profile"
  #endif
#endif

/* Automatically set up RTI_WIN32 flag if not defined based upon MSVC _WIN32 or WIN32 flags for Windows machines*/
#if (defined(_WIN32) || defined(WIN32)) && !defined(RTI_WINCE) && !defined(RTI_INTIME) && _MSC_VER > 1310 && !defined(RTI_WIN32)
  #define RTI_WIN32
#endif

/*
 * Automatically set up RTI_POSIX_THREADS for POSIX architectures:
 * AIX, Android, Darwin, Integrity, Linux, and QNX.
 * Note: Android is covered with RTI_LINUX specifically for POSIX threads
 * but not for POSIX in general.
 */
#if defined(RTI_AIX) || defined(RTI_DARWIN) || defined(RTI_INTY) \
        || defined(RTI_LINUX) || defined(RTI_QNX)
  #ifndef RTI_POSIX_THREADS
    #define RTI_POSIX_THREADS
  #endif
#endif

/*
 * Set up RTI_OSAPI_POSIX_RWLOCK_SUPPORTED for POSIX architectures but Darwin.
 * We don't support rwlock on BSD platforms because of:
 * https://www.gnu.org/software/gnulib/manual/html_node/pthread_005frwlock_005ftimedrdlock.html
 */
#if defined(RTI_POSIX_THREADS) && !defined(RTI_DARWIN)
  #ifndef RTI_OSAPI_POSIX_RWLOCK_SUPPORTED
    #define RTI_OSAPI_POSIX_RWLOCK_SUPPORTED
  #endif
#endif

/* Automatically set up RTI_ROBUST_POSIX_MUTEXES for cases where we use robust 
 * pthread mutexes instead of SemMutex (at the time of writing, only QNX) */
#if defined(RTI_QNX) && !defined(RTI_DISABLE_ROBUST_PTHREAD_MUTEX)
  #define RTI_ROBUST_POSIX_MUTEXES
#endif

/* Macro to check if GLIBC is at least a certain version */
#define GLIBC_AT_LEAST(major, minor) \
    ((__GLIBC__ > (major)) || ((__GLIBC__ == (major)) && (__GLIBC_MINOR__ >= (minor))))


#endif /* osapi_platform_h */
