/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)reda_hash_impl.h    generated by: makeheader    Sat Mar  9 09:37:41 2024
 *
 *		built from:	hash_impl.ifc
 */

#ifndef reda_hash_impl_h
#define reda_hash_impl_h


#ifdef __cplusplus
extern "C" {
#endif



#define RTI_ABS(x)      (((x) >= 0) ? (x) : (-(x)))

/*
 * ROBUSTNESS-1335: These macros cause overflows in operations along the
 * entire tree. However, the fix for these harmless, expected overflows,
 * adding some casts, causes a significant performance degradation. In order
 * to ignore these overflows and reduce the noise in UBSan analysis, we will
 * add the casts to these macros only when the plan is enabled.
 */
#ifdef RTI_ADDRESS_SANITIZER_IS_ENABLED
  #define REDAHash_hashSimpleIntMacro(key, numHashBuckets) \
    (RTI_ABS(*((int *) (key)) % (numHashBuckets)))

  #define REDAHash_hashSimpleDoubleIntMacro(key, numHashBuckets) \
    (int) (RTI_ABS( \
            ((long long) *((int *) (key)) + *(((int *) (key)) + 1)) \
            % (numHashBuckets)))

  #define REDAHash_hashSimpleTripleIntMacro(key, numHashBuckets) \
    (int) (RTI_ABS( \
            ((long long) *((int *) (key)) + *(((int *) (key)) + 1) \
                    + *(((int *) (key)) + 2)) \
            % (numHashBuckets)))

  #define REDAHash_hashSimpleQuadIntMacro(key, numHashBuckets) \
    (int) (RTI_ABS( \
            ((long long) *((int *) (key)) + *(((int *) (key)) + 1) \
                    + *(((int *) (key)) + 2) + *(((int *) (key)) + 3)) \
            % (numHashBuckets)))

  #define REDAHash_hashSimpleFiveIntMacro(key, numHashBuckets) \
    (int) (RTI_ABS( \
            ((long long) *((int *) (key)) + *(((int *) (key)) + 1) \
                    + *(((int *) (key)) + 2) + *(((int *) (key)) + 3) \
                    + *(((int *) (key)) + 4)) \
            % (numHashBuckets)))
#else
  #define REDAHash_hashSimpleIntMacro(key, numHashBuckets) \
    (RTI_ABS(*((int *) (key)) % (numHashBuckets)))

  #define REDAHash_hashSimpleDoubleIntMacro(key, numHashBuckets) \
    (RTI_ABS((*((int *) (key)) + *(((int *) (key)) + 1)) % (numHashBuckets)))

  #define REDAHash_hashSimpleTripleIntMacro(key, numHashBuckets) \
    (RTI_ABS((*((int *) (key)) + *(((int *) (key)) + 1) \
                    + *(((int *) (key)) + 2)) \
            % (numHashBuckets)))

  #define REDAHash_hashSimpleQuadIntMacro(key, numHashBuckets) \
    (RTI_ABS((*((int *) (key)) + *(((int *) (key)) + 1) \
                    + *(((int *) (key)) + 2) + *(((int *) (key)) + 3)) \
            % (numHashBuckets)))

  #define REDAHash_hashSimpleFiveIntMacro(key, numHashBuckets) \
    (RTI_ABS((*((int *) (key)) + *(((int *) (key)) + 1) \
                    + *(((int *) (key)) + 2) + *(((int *) (key)) + 3) \
                    + *(((int *) (key)) + 4)) \
            % (numHashBuckets)))
#endif

#define REDAHash_callFncOrUseMacro(hashDefinition, key) \
   ((hashDefinition->hashFunction == REDAHash_hashSimpleInt) ? \
    REDAHash_hashSimpleIntMacro(key, hashDefinition->numberOfBuckets) : \
    ((hashDefinition->hashFunction == REDAHash_hashSimpleDoubleInt) ? \
     REDAHash_hashSimpleDoubleIntMacro(key, hashDefinition->numberOfBuckets) : \
     ((hashDefinition->hashFunction == REDAHash_hashSimpleTripleInt) ? \
      REDAHash_hashSimpleTripleIntMacro(key, hashDefinition->numberOfBuckets) :\
      ((hashDefinition->hashFunction==REDAHash_hashSimpleQuadInt) ? \
       REDAHash_hashSimpleQuadIntMacro(key, hashDefinition->numberOfBuckets) : \
       ((hashDefinition->hashFunction==REDAHash_hashSimpleFiveInt) ? \
        REDAHash_hashSimpleFiveIntMacro(key, hashDefinition->numberOfBuckets) : \
        hashDefinition->hashFunction(key, hashDefinition->numberOfBuckets))))))

#define REDAHash_computeHash(hashDefinition, key) \
    (((hashDefinition)->numberOfBuckets <= 1) ? 0 : \
     REDAHash_callFncOrUseMacro((hashDefinition), (key)))


#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* reda_hash_impl_h */
