/*
 * (c) Copyright, Real-Time Innovations, 2011-2022.
 * All rights reserved.
 * No duplications, whole or partial, manual or electronic, may be made
 * without express written permission.  Any such copies, or
 * revisions thereof, must display this notice unaltered.
 * This code contains trade secrets of Real-Time Innovations, Inc.
 */

#ifndef rtitest_context_h
#define rtitest_context_h

#include "rtitest/test_log.h"
#include "rtitest/test_dll.h"

#ifdef __cplusplus
extern "C" {
#endif

#if defined(RTI_WINCE)
  #include <windows.h>
#endif /*#if defined(RTI_WINCE)*/

#define RTI_TEST_CONTEXT_IS_INITIALIZED 22

struct RTITestContext {
    int dummy;
    int isInitialized;
};

/**
 * @internal
 * @brief Creates a new context.
 * @return Newly created context.
 */
extern RTITestDllExport struct RTITestContext *RTITestContext_new(void);

/**
 * @internal
 * @brief Deletes an existing context.
 * @param self The context that needs to be deleted.
 */
extern RTITestDllExport void RTITestContext_delete(struct RTITestContext *self);

/** @internal
 * \ingroup RTITestContextClass
 * @brief Initializes a RTITestContext.
 * @param self The context that needs to be initialized.
 */
extern RTITestDllExport int RTITestContext_initialize(
        struct RTITestContext *self);

/** @internal
 * \ingroup RTITestContextClass
 * @brief Finallizes a RTITestContext.
 * @param self The context that needs to be finalized.
 */
extern RTITestDllExport void RTITestContext_finalize(
        struct RTITestContext *self);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* rtitest_context_h */
