/**
 * (c) Copyright, Real-Time Innovations, 2017-2017.
 * All rights reserved.
 * No duplications, whole or partial, manual or electronic, may be made
 * without express written permission.  Any such copies, or
 * revisions thereof, must display this notice unaltered.
 * This code contains trade secrets of Real-Time Innovations, Inc.
 *
 * Description
 * -----------
 * Property names and values for security authentication plugin
 * =====================================================================
 */
#ifndef security_authentication_h
#define security_authentication_h

#include "security/security_default.h"
#include "security/security_infrastructure.h"

#ifdef __cplusplus
    extern "C" {
#endif

/* DDS Security specification Table 45 */
#define RTI_SECURITY_AUTHENTICATION_IDENTITY_CLASS_ID \
    RTI_SECURITY_AUTHENTICATION_PLUGIN_CLASS ":1.0"

/* RTI Lightweight Security */
#define RTI_SECURITY_AUTHENTICATION_LIGHTWEIGHT_IDENTITY_CLASSID \
    RTI_SECURITY_AUTHENTICATION_LIGHTWEIGHT_SECURITY_PLUGIN_CLASS ":1.2"

/* Property names and values according to DDS Security specification */
#define DDS_SECURITY_AUTHENTICATION_PROPERTY_PREFIX \
    DDS_TRUST_PLUGIN_PROPERTY_PREFIX "auth."
#define DDS_SECURITY_IDENTITY_CA_PROPERTY \
    DDS_SECURITY_AUTHENTICATION_PROPERTY_PREFIX "identity_ca"
#define DDS_SECURITY_PRIVATE_KEY_PROPERTY \
    DDS_SECURITY_AUTHENTICATION_PROPERTY_PREFIX "private_key"
#define DDS_SECURITY_PASSWORD_PROPERTY \
    DDS_SECURITY_AUTHENTICATION_PROPERTY_PREFIX "password"
#define DDS_SECURITY_IDENTITY_CERTIFICATE_PROPERTY \
    DDS_SECURITY_AUTHENTICATION_PROPERTY_PREFIX "identity_certificate"

#define DDS_DOMAIN_PARTICIPANT_DOMAIN_TAG_PROPERTY \
    "dds.domain_participant.domain_tag"

/* Property names and values used in 5.3.0 and below */
#define RTI_SECURITY_AUTHENTICATION_PROPERTY_PREFIX "authentication."
#define RTI_SECURITY_CA_FILE_PROPERTY \
    RTI_SECURITY_AUTHENTICATION_PROPERTY_PREFIX "ca_file"
#define RTI_SECURITY_CERTIFICATE_FILE_PROPERTY \
    RTI_SECURITY_AUTHENTICATION_PROPERTY_PREFIX "certificate_file"
#define RTI_SECURITY_PRIVATE_KEY_FILE_PROPERTY \
    RTI_SECURITY_AUTHENTICATION_PROPERTY_PREFIX "private_key_file"

/* RTI-specific properties */
#define RTI_SECURITY_X509_VERIFY_LIBRARY_PROPERTY \
    RTI_SECURITY_AUTHENTICATION_PROPERTY_PREFIX "x509_verify.library"
#define RTI_SECURITY_X509_VERIFY_FUNCTION_NAME_PROPERTY \
    RTI_SECURITY_AUTHENTICATION_PROPERTY_PREFIX "x509_verify.function_name"
#define RTI_SECURITY_KEY_ESTABLISHMENT_ALGORITHM_PROPERTY \
    RTI_SECURITY_AUTHENTICATION_PROPERTY_PREFIX "key_establishment_algorithm"
#define RTI_SECURITY_SHARED_SECRET_ALGORITHM_PROPERTY \
    RTI_SECURITY_AUTHENTICATION_PROPERTY_PREFIX "shared_secret_algorithm"
#define RTI_SECURITY_DH_LEGACY_PROPERTY_VALUE "dh"
#define RTI_SECURITY_ECDH_LEGACY_PROPERTY_VALUE "ecdh"
#define RTI_SECURITY_ALTERNATIVE_CA_FILES_PROPERTY \
    RTI_SECURITY_AUTHENTICATION_PROPERTY_PREFIX "alternative_ca_files"
#define RTI_SECURITY_CRL_PROPERTY \
    RTI_SECURITY_AUTHENTICATION_PROPERTY_PREFIX "crl"
#define RTI_SECURITY_X509v3_EXTENSION_ENFORCEMENT_PREFIX \
    RTI_SECURITY_AUTHENTICATION_PROPERTY_PREFIX "x509v3_extension_enforcement."
#define RTI_SECURITY_KEY_USAGE_PROPERTY \
    RTI_SECURITY_X509v3_EXTENSION_ENFORCEMENT_PREFIX "key_usage"
#define RTI_SECURITY_PRIVATE_KEY_FORM_PROPERTY \
    RTI_SECURITY_AUTHENTICATION_PROPERTY_PREFIX "keyform"
#define RTI_SECURITY_RSA_PSS_PAD_PROPERTY \
    RTI_SECURITY_AUTHENTICATION_PROPERTY_PREFIX "rsa_pss_pad"
#define RTI_SECURITY_PROPAGATE_SIMPLIFIED_IDENTITY_CERTIFICATE_PROPERTY \
    RTI_SECURITY_AUTHENTICATION_PROPERTY_PREFIX "propagate_simplified_identity_certificate"
#define RTI_SECURITY_PARTICIPANT_DISCOVERY_PROTECTION_KEY \
    RTI_SECURITY_AUTHENTICATION_PROPERTY_PREFIX "participant_discovery_protection_key"
#define RTI_SECURITY_PARTICIPANT_CDS_NAME "dds.participant.cds_name"
#define RTI_SECURITY_ENABLE_OCSP_PROPERTY \
    RTI_SECURITY_AUTHENTICATION_PROPERTY_PREFIX "enable_ocsp"
#define RTI_SECURITY_OCSP_STAPLING_PROPERTY \
    RTI_SECURITY_AUTHENTICATION_PROPERTY_PREFIX "ocsp_stapling"
#define RTI_SECURITY_OCSP_TEST_PORT_PROPERTY \
    RTI_SECURITY_AUTHENTICATION_PROPERTY_PREFIX "ocsp_test_port"
#define RTI_SECURITY_ENABLE_DISCOVERY_SUBJECT_NAME_PROPAGATION \
    RTI_SECURITY_AUTHENTICATION_PROPERTY_PREFIX "enable_discovery_subject_name_propagation"

/* Backward-compatibility properties */
#define RTI_SECURITY_CRL_FILE_PROPERTY \
    RTI_SECURITY_AUTHENTICATION_PROPERTY_PREFIX "crl_file"
#define RTI_SECURITY_X509v3_EXTENSION_ENFORCEMENT_PREFIX \
    RTI_SECURITY_AUTHENTICATION_PROPERTY_PREFIX "x509v3_extension_enforcement."
#define RTI_SECURITY_KEY_USAGE_PROPERTY \
    RTI_SECURITY_X509v3_EXTENSION_ENFORCEMENT_PREFIX "key_usage"
#define RTI_SECURITY_PRIVATE_KEY_FORM_PROPERTY \
    RTI_SECURITY_AUTHENTICATION_PROPERTY_PREFIX "keyform"

/* Default property values */
#define RTI_SECURITY_PROPAGATE_SIMPLIFIED_IDENTITY_CERTIFICATE_PROPERTY_DEFAULT \
    RTI_TRUE

/*
 * Digital signature algorithm property values.
 * DDS Security specification Table 38.
 */
#define RTI_SECURITY_AUTHENTICATION_DSIGN_ALGO_RSASSA_PSS_MGF1SHA256_2048_SHA256 \
    "RSASSA-PSS-MGF1SHA256+2048+SHA256"
#define RTI_SECURITY_AUTHENTICATION_DSIGN_ALGO_RSASSA_PKCS1_V1_5_2048_SHA256 \
    "RSASSA-PKCS1-V1_5+2048+SHA256"
#define RTI_SECURITY_AUTHENTICATION_DSIGN_ALGO_ECDSA256_SHA256 \
    "ECDSA+P256+SHA256"
#define RTI_SECURITY_AUTHENTICATION_DSIGN_ALGO_ECDSA384_SHA384 \
    "ECDSA+P384+SHA384"
#define RTI_SECURITY_AUTHENTICATION_DSIGN_ALGO_EDDSA_ED25519_SHA512 \
    "EDDSA+ED25519+SHA512"
#define RTI_SECURITY_AUTHENTICATION_DSIGN_ALGO_EDDSA_ED448_SHAKE256 \
    "EDDSA+ED448+SHAKE256"

#define RTI_SECURITY_AUTHENTICATION_DSIGN_ALGO_RSA_LEGACY "RSASSA-PSS-SHA256"
#define RTI_SECURITY_AUTHENTICATION_DSIGN_ALGO_ECDSA_LEGACY "ECDSA-SHA256"

/* Key establishment algorithm property values */
#define RTI_SECURITY_AUTHENTICATION_KEST_ALGO_DHE "DHE+MODP-2048-256"
#define RTI_SECURITY_AUTHENTICATION_KEST_ALGO_ECDHE_P256 "ECDHE-CEUM+P256"
#define RTI_SECURITY_AUTHENTICATION_KEST_ALGO_ECDHE_P384 "ECDHE-CEUM+P384"
#define RTI_SECURITY_AUTHENTICATION_KEST_ALGO_ECDHE_X25519 "ECDHE-CEUM+X25519"
#define RTI_SECURITY_AUTHENTICATION_KEST_ALGO_ECDHE_X448 "ECDHE-CEUM+X448"

#define RTI_SECURITY_AUTHENTICATION_KEST_ALGO_DHE_LEGACY "DH+MODP-2048-256"
#define RTI_SECURITY_AUTHENTICATION_KEST_ALGO_ECDHE_LEGACY "ECDH+prime256v1-CEUM"

/*
 * EXFLAG_KUSAGE, which is defined by OpenSSL has been detected as a signed int,
 * but according to the documentation it's a uint32_t. Since we do bitwise
 * operations with this value some issues could arise (see SEC-2130). As a
 * workaround we redefine this flag so that it is detected now as unsigned.
 */
#define RTI_EXFLAG_KUSAGE ((unsigned int) EXFLAG_KUSAGE)

#ifdef __cplusplus
}    /* extern "C" */
#endif

#endif /* security_authentication_h */
