/**
 * (c) Copyright, Real-Time Innovations, 2021
 * All rights reserved.
 * No duplications, whole or partial, manual or electronic, may be made
 * without express written permission.  Any such copies, or
 * revisions thereof, must display this notice unaltered.
 * This code contains trade secrets of Real-Time Innovations, Inc.
 *
 * Description
 * -----------
 * RTI definitions of cryptography library structures and macros to enable
 * supporting for OpenSSL 1.1.0 (or lower).
 * A user application does not need to include this header file. It's only
 * intended for internal use by other connextdds modules.
 * =========================================================================
 */

#ifndef security_crypto_lib_adapter_openssl102_h
#define security_crypto_lib_adapter_openssl102_h

#if !defined(RTI_WOLFSSL) \
    && (OPENSSL_VERSION_NUMBER < RTI_OPENSSL_VERSION_1_1_0)

  #ifdef RTI_SECURITY_PLUGINS_SUPPORT_THREAD_SPECIFIC_STORAGE
    #undef RTI_SECURITY_PLUGINS_SUPPORT_THREAD_SPECIFIC_STORAGE
  #endif

  #define RTI_OPENSSL_LOCKING

  /* ***************************************************************************
   * EVP_CIPHER_CTX
   * **************************************************************************/
  /*
   * Cipher context was public in openSSL < 1.1.0. Also, the behavior for init
   * was different than in later versions.
   */
  #define RTI_Security_CryptoLibAdapterEvpCipherCtxHandle EVP_CIPHER_CTX

  #define RTI_Security_CryptoLibAdapterEvpCipherCtxHandle_getCryptoLibCtx(ctxHandle__) \
      (ctxHandle__)

  #define RTI_Security_CryptoLibAdapterEvpCipherCtxHandle_initialize( \
          ctxHandle__, \
          cryptoContext__) \
      (EVP_CIPHER_CTX_init(ctxHandle__), (1))

  #define RTI_Security_CryptoLibAdapterEvpCipherCtxHandle_reset(ctxHandle__) \
      EVP_CIPHER_CTX_cleanup(ctxHandle__); \
      EVP_CIPHER_CTX_init(ctxHandle__)

  #define RTI_Security_CryptoLibAdapterEvpCipherCtxHandle_finalize( \
          ctxHandle__, \
          cryptoContext__) \
      RTI_Security_Utility_unusedParameter(cryptoContext__); \
      EVP_CIPHER_CTX_cleanup(ctxHandle__)

  /* ***************************************************************************
   * EVP_MD_CTX
   * **************************************************************************/
  /*
   * MD context was public in openSSL < 1.1.0. Also, the behavior for init
   * was different than in later versions.
   */
  #define RTI_Security_CryptoLibAdapterEvpMdCtxHandle EVP_MD_CTX
  #define RTI_Security_CryptoLibAdapterEvpMdCtxHandle_getCryptoLibCtx(ctxHandle__) (ctxHandle__)
  #define RTI_Security_CryptoLibAdapterEvpMdCtxHandle_initialize( \
          ctxHandle__, \
          cryptoContext__) \
      (EVP_MD_CTX_init(ctxHandle__), (1))
  #define RTI_Security_CryptoLibAdapterEvpMdCtxHandle_finalize( \
          ctxHandle__, \
          cryptoContext__) \
      RTI_Security_Utility_unusedParameter(cryptoContext__); \
      EVP_MD_CTX_cleanup(ctxHandle__)
  #define RTI_Security_CryptoLibAdapterEvpMdCtxHandle_reset(ctxHandle__) \
      EVP_MD_CTX_cleanup(ctxHandle__); \
      EVP_MD_CTX_init(ctxHandle__)

  /* ***************************************************************************
   * EVP_ENCODE_CTX
   * **************************************************************************/
  /*
   * ENCODE context was public in openSSL < 1.1.0. The behavior has not
   * changed. EvpEncodeCtx does not require a separate initializer function,
   * as EVP_DecodeInit takes care of that.
   */
  #define RTI_Security_CryptoLibAdapterEvpEncodeCtxHandle EVP_ENCODE_CTX

  #define RTI_Security_CryptoLibAdapterEvpEncodeCtxHandle_initialize( \
          ctxHandle__) \
     (1)

  #define RTI_Security_CryptoLibAdapterEvpEncodeCtxHandle_finalize( \
          ctxHandle__) \
      RTI_Security_Utility_unusedParameter(ctxHandle__)

  #define RTI_Security_CryptoLibAdapterEvpEncodeCtxHandle_getCryptoLibCtx( \
          ctxHandle__) \
      (ctxHandle__)

  /* ***************************************************************************
   * EVP_PKEY_new_mac_key
   * **************************************************************************/
  #define RTI_Security_CryptoLibAdapterEvpNewMacKey \
      EVP_PKEY_new_mac_key

#endif /* !defined(RTI_WOLFSSL) && (OPENSSL_VERSION_NUMBER < RTI_OPENSSL_VERSION_1_1_0) */
#endif /* security_crypto_lib_adapter_openssl102_h */
