/*
 * (c) Copyright, Real-Time Innovations, 2023.
 * All rights reserved.
 *
 * No duplications, whole or partial, manual or electronic, may be made
 * without express written permission.  Any such copies, or
 * revisions thereof, must display this notice unaltered.
 * This code contains trade secrets of Real-Time Innovations, Inc.
 */

#ifndef transport_transportTest_h
#define transport_transportTest_h

#include "rtitest/test_setting.h"
#include "transport/transport_common.h"
#include "transport/transport_dll.h"

#ifdef __cplusplus
extern "C" {
#endif

extern NDDS_TransportTEST_DllExport RTIBool
NDDS_Transport_UDPv6_Tester_checkForIPv6Support(void);

/**
 * Indicates the maximum amount of times NDDS_TransportHelper_getUsablePort can
 * be called before reseting the port that is returned.
 */
#define TRANSPORT_TEST_MAX_USABLE_PORT_REQUEST (40)

/**
 * @brief Get a usable Port from the list of well known ports. The function will
 *        calculate the port based on the arguments, and will return a different
 *        port based on the number of times this funtion is called. Once the
 *        function is called TRANSPORT_TEST_MAX_USABLE_PORT_REQUEST times, it
 *        start returning the same port returned the first time it was called.
 *        For example if we call this function with the following arguments:
 *                   domainId=12, portBase=7400, and domainIdGain=250.
 *        The first time is called it will return 10399, the second time 10398,
 *        and once it reaches the maximum request it return 10399 again.
 * @param domainId the domain we are using to run the tests.
 * @param portBase the well known port_base. We need to pass it because the
 *                 structure is defined in dds_c.1.0
 * @param domainIdGain the well known ddomain_id_gain. We need to pass it
 *                     because the structure is defined in dds_c.1.0
 * @return the usable Port from the list of well known ports
 */
extern NDDS_TransportTEST_DllExport NDDS_Transport_Port_t NDDS_TransportHelper_getUsablePort(
        RTI_INT32 domainId,
        RTI_INT32 portBase,
        RTI_INT32 domainIdGain);

#ifdef __cplusplus
}
#endif

#endif /* transport_transportTest_h */