/*
 * (c) Copyright, Real-Time Innovations, 2022.
 * All rights reserved.
 *
 * No duplications, whole or partial, manual or electronic, may be made
 * without express written permission.  Any such copies, or
 * revisions thereof, must display this notice unaltered.
 * This code contains trade secrets of Real-Time Innovations, Inc.
 */

#ifndef HTTP_RESPONSE_HPP
#define HTTP_RESPONSE_HPP

#include <string>
#include <unordered_map>

namespace rti { namespace apputils { namespace http {

/**
 * @brief This class contains all the elements involved in an HTTP response
 */
struct Response {
    /**
     * Response's status code
     */
    unsigned short http_status_code;

    /**
     * Response's reason phrase
     */
    std::string http_reason_phrase;

    /**
     * Response's content type
     */
    std::string http_content_type;

    /**
     * Response's body
     */
    std::string http_content;

    /**
     * Response's headers as a map
     */
    std::unordered_map<std::string, std::string> http_headers;

};
}}}  // namespace rti::apputils::http

#endif  // HTTP_RESPONSE_HPP
