/*
 * (c) Copyright, Real-Time Innovations, 2016-.
 * All rights reserved.
 * No duplications, whole or partial, manual or electronic, may be made
 * without express written permission.  Any such copies, or
 * revisions thereof, must display this notice unaltered.
 * This code contains trade secrets of Real-Time Innovations, Inc.
 */

#ifndef HPP_RTI_RECORDING_STORAGE_ESTORAGEFORMAT_HPP_
#define HPP_RTI_RECORDING_STORAGE_ESTORAGEFORMAT_HPP_

#include <stdint.h>

#include "reda/reda_string.h"
#include "dds_c/dds_c_infrastructure.h"

#include "dds/core/Exception.hpp"

namespace rti { namespace recording { namespace storage {

class EStorageFormat {
public:

    static const uint16_t INVALID_REPRESENTATION =
            RTI_CDR_ENCAPSULATION_ID_INVALID;

    static const EStorageFormat XCDR() {
        static const EStorageFormat XCDR(
                0,
                "XCDR",
                DDS_XCDR_DATA_REPRESENTATION);
        return XCDR;
    }
    static const EStorageFormat XCDR2() {
        static const EStorageFormat XCDR2(
                1,
                "XCDR2",
                DDS_XCDR2_DATA_REPRESENTATION);
        return XCDR2;
    }
    static const EStorageFormat XCDR_AUTO() {
        static const EStorageFormat XCDR_AUTO(
                2,
                "XCDR_AUTO",
                DDS_AUTO_DATA_REPRESENTATION);
        return XCDR_AUTO;
    }
    static const EStorageFormat JSON_SQLITE() {
        static const EStorageFormat JSON_SQLITE(
                3,
                "JSON_SQLITE",
                INVALID_REPRESENTATION);
        return JSON_SQLITE;
    }
    /**
     * This format is only used by the legacy DB plugins, for the moment.
     */
    static const EStorageFormat DESERIALIZED() {
        static const EStorageFormat DESERIALIZED(
                4,
                "DESERIALIZED",
                INVALID_REPRESENTATION);
        return DESERIALIZED;
    }

    static EStorageFormat from_string_ignore_case(const std::string& value)
    {
        if (REDAString_iCompare(
                value.c_str(),
                XCDR().enum_value().c_str()) == 0) {
            return XCDR();
        } else if (REDAString_iCompare(
                value.c_str(),
                XCDR2().enum_value().c_str()) == 0) {
            return XCDR2();
        } else if (REDAString_iCompare(
                value.c_str(),
                XCDR_AUTO().enum_value().c_str()) == 0) {
            return XCDR_AUTO();
        } else if (REDAString_iCompare(
                value.c_str(),
                JSON_SQLITE().enum_value().c_str()) == 0) {
            return JSON_SQLITE();
        } else {
            throw dds::core::UnsupportedError(
                    "Unrecognized DB Format value, must be one of "
                    "{XCDR,XCDR2,XCDR_AUTO,JSON_SQLITE}");
        }
    }

    EStorageFormat(const EStorageFormat& other)
            : enum_value_(other.enum_value_),
              enum_string_(other.enum_string_),
              data_rep_id_(other.data_rep_id_){}

    void operator =(const EStorageFormat& other)
    {
        enum_value_ = other.enum_value_;
        enum_string_ = other.enum_string_;
        data_rep_id_ = other.data_rep_id_;
    }

    bool operator ==(const EStorageFormat& other) const
    {
        return enum_value_ == other.enum_value_;
    }

    bool operator !=(const EStorageFormat& other) const
    {
        return enum_value_ != other.enum_value_;
    }

    uint32_t value() const { return enum_value_; }

    const std::string& enum_value() const { return enum_string_; }

    /**
     * This function returns the associated data representation when it applies.
     * In the cases it doesn't, it returns the value INVALID_REPRESENTATION
     */
    DDS_DataRepresentationId_t data_representation() const
    {
        return data_rep_id_;
    }

private:
    uint32_t enum_value_;
    std::string enum_string_;
    DDS_DataRepresentationId_t data_rep_id_;

    EStorageFormat(
            const uint32_t value,
            const std::string& enum_string,
            const DDS_DataRepresentationId_t rep_id) :
        enum_value_(value),
        enum_string_(enum_string),
        data_rep_id_(rep_id) {}

};

} } } /* namespace rti::recording::storage */

#endif /* HPP_RTI_RECORDING_STORAGE_ESTORAGEFORMAT_HPP_ */
